"""Sentence models for evaluation on the Russian part of MTEB"""

import torch

from mteb.models.instruct_wrapper import InstructSentenceTransformerModel
from mteb.models.model_meta import (
    ModelMeta,
    ScoringFunction,
)
from mteb.models.sentence_transformer_wrapper import sentence_transformers_loader
from mteb.types import PromptType

from .bge_models import bge_m3_training_data
from .nomic_models import (
    nomic_training_data,
)

GIGA_task_prompts = {
    "TERRa": "Given a premise, retrieve a hypothesis that is entailed by the premise",
    "RuSTSBenchmarkSTS": "Retrieve semantically similar text",
    "RUParaPhraserSTS": "Retrieve semantically similar text",
    "CEDRClassification": "Дан комментарий, определи выраженную в нем эмоцию (радость, грусть, удивление, страх, гнев или нейтрально)",
    "GeoreviewClassification": "Classify the organization rating based on the reviews",
    "GeoreviewClusteringP2P": "Классифицируй рейтинг организации на основе отзыва",
    "HeadlineClassification": "Классифицируй тему данного новостного заголовка",
    "InappropriatenessClassification": "Классифицируй данный комментарий как токсичный или не токсичный",
    "KinopoiskClassification": "Classify the sentiment expressed in the given movie review text",
    "RuReviewsClassification": "Classify product reviews into positive, negative or neutral sentiment",
    "RuSciBenchGRNTIClassification": "Classify the category of scientific papers based on the titles and abstracts",
    "RuSciBenchGRNTIClusteringP2P": "Классифицируй категорию научной статьи основываясь на аннотации",
    "RuSciBenchOECDClassification": "Classify the category of scientific papers based on the titles and abstracts",
    "RuSciBenchOECDClusteringP2P": "Классифицируй категорию научной статьи основываясь на аннотации",
    "SensitiveTopicsClassification": "Классифицируй чувствительную тему по запросу",
    "RuBQRetrieval": {
        "query": "Given a question, retrieve Wikipedia passages that answer the question",
        "document": "",
    },
    "RuBQReranking": {
        "query": "Given a question, retrieve Wikipedia passages that answer the question",
        "document": "",
    },
    "RiaNewsRetrieval": {
        "query": "Given a news title, retrieve relevant news article",
        "document": "",
    },
    "MIRACLReranking": {
        "query": "Given a question, retrieve Wikipedia passages that answer the question",
        "document": "",
    },
    "MIRACLRetrieval": {
        "query": "Given a question, retrieve Wikipedia passages that answer the question",
        "document": "",
    },
    "ArguAna": {
        "query": "Given a search query, retrieve passages that answer the question",
        "document": "Given a search query, retrieve passages that answer the question",
    },
    "CQADupstackAndroidRetrieval": {
        "query": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
        "document": "",
    },
    "CQADupstackEnglishRetrieval": {
        "query": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
        "document": "",
    },
    "CQADupstackGamingRetrieval": {
        "query": "Given a search query, retrieve passages that answer the question",
        "document": "Given a search query, retrieve passages that answer the question",
    },
    "CQADupstackGisRetrieval": {
        "query": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
        "document": "",
    },
    "CQADupstackMathematicaRetrieval": {
        "query": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
        "document": "",
    },
    "CQADupstackPhysicsRetrieval": {
        "query": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
        "document": "",
    },
    "CQADupstackProgrammersRetrieval": {
        "query": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
        "document": "",
    },
    "CQADupstackStatsRetrieval": {
        "query": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
        "document": "",
    },
    "CQADupstackTexRetrieval": {
        "query": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
        "document": "",
    },
    "CQADupstackUnixRetrieval": {
        "query": "Given a search query, retrieve passages that answer the question",
        "document": "Given a search query, retrieve passages that answer the question",
    },
    "CQADupstackWebmastersRetrieval": {
        "query": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
        "document": "",
    },
    "CQADupstackWordpressRetrieval": {
        "query": "Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question",
        "document": "",
    },
    "ClimateFEVER": {
        "query": "Given a claim about climate change, retrieve documents that support or refute the claim",
        "document": "",
    },
    "ClimateFEVERHardNegatives": {
        "query": "Given a search query, retrieve passages that answer the question",
        "document": "",
    },
    "DBPedia": {
        "query": "Given a query, retrieve relevant entity descriptions from DBPedia",
        "document": "",
    },
    "FEVER": {
        "query": "Given a claim, retrieve documents that support or refute the claim",
        "document": "",
    },
    "FEVERHardNegatives": {
        "query": "Given a search query, retrieve passages that answer the question",
        "document": "",
    },
    "FiQA2018": {
        "query": "Given a web search query, retrieve relevant passages that answer the query",
        "document": "",
    },
    "HotpotQA": {
        "query": "Given a multi-hop question, retrieve documents that can help answer the question",
        "document": "",
    },
    "HotpotQAHardNegatives": {
        "query": "Given a search query, retrieve passages that answer the question",
        "document": "",
    },
    "MSMARCO": {
        "query": "Given a web search query, retrieve relevant passages that answer the query",
        "document": "",
    },
    "NFdocument": {
        "query": "Given a question, retrieve relevant documents that best answer the question",
        "document": "",
    },
    "NQ": {
        "query": "Given a question, retrieve Wikipedia passages that answer the question",
        "document": "",
    },
    "QuoraRetrieval": {
        "query": "Given a question, retrieve questions that are semantically equivalent to the given question",
        "document": "",
    },
    "SCIDOCS": {
        "query": "Given a search query, retrieve passages that answer the question",
        "document": "",
    },
    "SciFact": {
        "query": "Given a scientific claim, retrieve documents that support or refute the claim",
        "document": "",
    },
    "TRECCOVID": {
        "query": "Given a search query, retrieve passages that answer the question",
        "document": "",
    },
    "Touche2020": {
        "query": "Given a question, retrieve detailed and persuasive arguments that answer the question",
        "document": "",
    },
    "Touche2020Retrieval.v3": {
        "query": "Given a search query, retrieve passages that answer the question",
        "document": "",
    },
    "BIOSSES": "Retrieve semantically similar text",
    "SICK-R": "Retrieve semantically similar text",
    "STS12": "Retrieve semantically similar text",
    "STS13": "Retrieve semantically similar text",
    "STS14": "Retrieve semantically similar text",
    "STS15": "Retrieve semantically similar text",
    "STS16": "Retrieve semantically similar text",
    "STS17": "Retrieve semantically similar text",
    "STS22": "Retrieve semantically similar text",
    "STS22.v2": "Retrieve semantically similar text",
    "STSBenchmark": "Retrieve semantically similar text",
    "SummEval": "Given a news summary, retrieve other semantically similar summaries",
    "SummEvalSummarization.v2": "Given a news summary, retrieve other semantically similar summaries",
    "AmazonCounterfactualClassification": "Classify a given Amazon customer review text as either counterfactual or not-counterfactual",
    "AmazonPolarityClassification": "Classify Amazon reviews into positive or negative sentiment",
    "AmazonReviewsClassification": "Classify the given Amazon review into its appropriate rating category",
    "Banking77Classification": "Given a online banking query, find the corresponding intents",
    "EmotionClassification": "Classify the emotion expressed in the given Twitter message into one of the six emotions: anger, fear, joy, love, sadness, and surprise",
    "ImdbClassification": "Classify the sentiment expressed in the given movie review text from the IMDB dataset",
    "MassiveIntentClassification": "Given a user utterance as query, find the user intents",
    "MassiveScenarioClassification": "Given a user utterance as query, find the user scenarios",
    "MTOPDomainClassification": "Classify the intent domain of the given utterance in task-oriented conversation",
    "MTOPIntentClassification": "Classify the intent of the given utterance in task-oriented conversation",
    "ToxicConversationsClassification": "Classify the given comments as either toxic or not toxic",
    "TweetSentimentExtractionClassification": "Classify the sentiment of a given tweet as either positive, negative, or neutral",
    "ArxivClusteringP2P": "Identify the main and secondary category of Arxiv papers based on the titles and abstracts",
    "ArxivClusteringS2S": "Identify the main and secondary category of Arxiv papers based on the titles",
    "ArXivHierarchicalClusteringP2P": "Identify the main and secondary category of Arxiv papers based on the titles and abstracts",
    "ArXivHierarchicalClusteringS2S": "Identify the main and secondary category of Arxiv papers based on the titles",
    "BiorxivClusteringP2P": "Identify the main category of Biorxiv papers based on the titles and abstracts",
    "BiorxivClusteringS2S": "Identify the main category of Biorxiv papers based on the titles",
    "BiorxivClusteringP2P.v2": "Identify the main category of Biorxiv papers based on the titles and abstracts",
    "MedrxivClusteringP2P": "Identify the main category of Medrxiv papers based on the titles and abstract",
    "MedrxivClusteringS2S": "Identify the main category of Medrxiv papers based on the titles",
    "MedrxivClusteringP2P.v2": "Identify the main category of Medrxiv papers based on the titles and abstract",
    "MedrxivClusteringS2S.v2": "Identify the main category of Medrxiv papers based on the titles",
    "RedditClustering": "Identify the topic or theme of Reddit posts based on the titles",
    "RedditClusteringP2P": "Identify the topic or theme of Reddit posts based on the titles and posts",
    "StackExchangeClustering": "Identify the topic or theme of StackExchange posts based on the titles",
    "StackExchangeClusteringP2P": "Identify the topic or theme of StackExchange posts based on the given paragraphs",
    "StackExchangeClustering.v2": "Identify the topic or theme of StackExchange posts based on the titles",
    "StackExchangeClusteringP2P.v2": "Identify the topic or theme of StackExchange posts based on the given paragraphs",
    "TwentyNewsgroupsClustering": "Identify the topic or theme of the given news articles",
    "TwentyNewsgroupsClustering.v2": "Identify the topic or theme of the given news articles",
    "AskUbuntuDupQuestions": {
        "query": "Retrieve duplicate questions from AskUbuntu forum",
        "document": "Retrieve duplicate questions from AskUbuntu forum",
    },
    "MindSmallReranking": "Given a search query, retrieve passages that answer the question",
    "SciDocsRR": "Given a title of a scientific paper, retrieve the titles of other relevant papers",
    "StackOverflowDupQuestions": "Retrieve duplicate questions from StackOverflow forum",
    "SprintDuplicateQuestions": "Retrieve duplicate questions from Sprint forum",
    "TwitterSemEval2015": "Retrieve tweets that are semantically similar to the given tweet",
    "TwitterURLCorpus": "Retrieve tweets that are semantically similar to the given tweet",
}

rubert_tiny = ModelMeta(
    loader=sentence_transformers_loader,
    name="cointegrated/rubert-tiny",
    languages=["rus-Cyrl"],
    open_weights=True,
    revision="5441c5ea8026d4f6d7505ec004845409f1259fb1",
    release_date="2021-05-24",
    n_parameters=11_900_000,
    memory_usage_mb=45,
    embed_dim=312,
    license="mit",
    max_tokens=512,
    reference="https://huggingface.co/cointegrated/rubert-tiny",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch"],
    use_instructions=False,
    public_training_code="https://gist.github.com/avidale/7bc6350f26196918bf339c01261f5c60",
    training_datasets={
        # [Yandex Translate corpus](https://translate.yandex.ru/corpus), [OPUS-100](https://huggingface.co/datasets/opus100)
        "Tatoeba",
    },
    adapted_from="google-bert/bert-base-multilingual-cased",
    public_training_data=None,
)

rubert_tiny2 = ModelMeta(
    loader=sentence_transformers_loader,
    name="cointegrated/rubert-tiny2",
    languages=["rus-Cyrl"],
    open_weights=True,
    revision="dad72b8f77c5eef6995dd3e4691b758ba56b90c3",
    release_date="2021-10-28",
    n_parameters=29_400_000,
    memory_usage_mb=112,
    embed_dim=312,
    license="mit",
    max_tokens=2048,
    reference="https://huggingface.co/cointegrated/rubert-tiny2",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch"],
    use_instructions=False,
    public_training_code="https://colab.research.google.com/drive/1mSWfIQ6PIlteLVZ9DKKpcorycgLIKZLf?usp=sharing",
    training_datasets=set(
        # https://huggingface.co/datasets/cointegrated/ru-paraphrase-NMT-Leipzig
        # Wikipedia https://huggingface.co/datasets/Madjogger/JamSpell_dataset
        # https://huggingface.co/datasets/imvladikon/leipzig_corpora_collection
    ),
    adapted_from="cointegrated/rubert-tiny",
    public_training_data=None,
)

sbert_large_nlu_ru = ModelMeta(
    loader=sentence_transformers_loader,
    name="ai-forever/sbert_large_nlu_ru",
    languages=["rus-Cyrl"],
    open_weights=True,
    revision="af977d5dfa46a3635e29bf0ef383f2df2a08d47a",
    release_date="2020-11-20",
    n_parameters=427_000_000,
    memory_usage_mb=1629,
    embed_dim=1024,
    license="mit",
    max_tokens=512,  # best guess
    reference="https://huggingface.co/ai-forever/sbert_large_nlu_ru",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    adapted_from="google/bert_uncased_L-12_H-768_A-12",
    training_datasets=set(
        # SNLI
        # MNLI
    ),
)

sbert_large_mt_nlu_ru = ModelMeta(
    loader=sentence_transformers_loader,
    name="ai-forever/sbert_large_mt_nlu_ru",
    languages=["rus-Cyrl"],
    open_weights=True,
    revision="05300876c2b83f46d3ddd422a7f17e45cf633bb0",
    release_date="2021-05-18",
    n_parameters=427_000_000,
    memory_usage_mb=1629,
    embed_dim=1024,
    license="not specified",
    max_tokens=512,  # best guess
    reference="https://huggingface.co/ai-forever/sbert_large_mt_nlu_ru",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets=set(
        # SNLI, MNLI
        # https://github.com/brmson/dataset-sts
    ),
)

user_base_ru = ModelMeta(
    loader=sentence_transformers_loader,
    loader_kwargs=dict(
        model_prompts={"query": "query: ", "document": "passage: "},
    ),
    name="deepvk/USER-base",
    languages=["rus-Cyrl"],
    open_weights=True,
    revision="436a489a2087d61aa670b3496a9915f84e46c861",
    release_date="2024-06-10",
    n_parameters=427_000_000,
    memory_usage_mb=473,
    embed_dim=768,
    license="apache-2.0",
    max_tokens=512,
    reference="https://huggingface.co/deepvk/USER-base",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch"],
    adapted_from="https://huggingface.co/deepvk/deberta-v1-base",
    use_instructions=True,
    citation="""@misc{deepvk2024user,
        title={USER: Universal Sentence Encoder for Russian},
        author={Malashenko, Boris and  Zemerov, Anton and Spirin, Egor},
        url={https://huggingface.co/datasets/deepvk/USER-base},
        publisher={Hugging Face}
        year={2024},
    }
    """,
    training_datasets={
        "BibleNLPBitextMining",
        # https://github.com/unicamp-dl/mMARCO
        # deepvk/ru-HNP
        # deepvk/ru-WANLI
        # MedNLI
        # RCB
        "TERRa",
        # Tapaco
        # Opus100
        # BiblePar
        # RudetoxifierDataDetox
        # RuParadetox
        "MIRACL",
        # MLDR
        # Lenta
        "MLSUMClusteringP2P",
        "MLSUMClusteringP2P.v2",
        "MLSUMClusteringS2S",
        "MLSUMClusteringS2S.v2",
        "MrTidyRetrieval",
        # "Panorama"
        # PravoIsrael
        # xlsum
        # Fialka-v1
        # RussianKeywords
        # Gazeta
        # Gsm8k-ru
        # DSumRu
        # SummDialogNews
    },
    public_training_code=None,
    public_training_data=None,
)

user_bge_m3 = ModelMeta(
    loader=sentence_transformers_loader,
    name="deepvk/USER-bge-m3",
    languages=["rus-Cyrl"],
    open_weights=True,
    revision="0cc6cfe48e260fb0474c753087a69369e88709ae",
    release_date="2024-07-05",
    n_parameters=359_026_688,
    memory_usage_mb=1370,
    embed_dim=1024,
    license="apache-2.0",
    max_tokens=8194,
    reference="https://huggingface.co/deepvk/USER-base",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch"],
    adapted_from="BAAI/bge-m3",
    use_instructions=False,
    training_datasets={
        "BibleNLPBitextMining",
        "MLSUMClusteringP2P",
        "MLSUMClusteringP2P.v2",
        "MLSUMClusteringS2S",
        "MLSUMClusteringS2S.v2",
        # not MTEB:
        # "deepvk/ru-HNP",
        # "deepvk/ru-WANLI",
        # "Shitao/bge-m3-data",
        # "RussianNLP/russian_super_glue",
        # "reciTAL/mlsum",
        # "Helsinki-NLP/opus-100",
        # "Helsinki-NLP/bible_para",
        # "d0rj/rudetoxifier_data_detox",
        # "s-nlp/ru_paradetox",
        # "Milana/russian_keywords",
        # "IlyaGusev/gazeta",
        # "d0rj/gsm8k-ru",
        # "bragovo/dsum_ru",
        # "CarlBrendt/Summ_Dialog_News",
    },
    public_training_code=None,
    public_training_data=None,
)

deberta_v1_ru = ModelMeta(
    loader=sentence_transformers_loader,
    name="deepvk/deberta-v1-base",
    languages=["rus-Cyrl"],
    open_weights=True,
    revision="bdd30b0e19757e6940c92c7aff19e8fc0a60dff4",
    release_date="2023-02-07",
    n_parameters=124_000_000,
    memory_usage_mb=473,
    embed_dim=768,
    license="apache-2.0",
    max_tokens=512,
    reference="https://huggingface.co/deepvk/deberta-v1-base",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch"],
    use_instructions=False,
    # Wikipedia, Books, Twitter comments, Pikabu, Proza.ru, Film subtitles, News websites, and Social corpus
    public_training_code=None,
    public_training_data=None,
    training_datasets={
        # 400 GB of filtered and deduplicated texts in total.
        # A mix of the following data: Wikipedia, Books, Twitter comments, Pikabu, Proza.ru,
        # Film subtitles, News websites, and Social corpus.
        # wikipedia
        "WikipediaRetrievalMultilingual",
        "WikipediaRerankingMultilingual",
        "RiaNewsRetrieval",  # probably
    },
)

rubert_base_cased = ModelMeta(
    loader=sentence_transformers_loader,
    name="DeepPavlov/rubert-base-cased",
    languages=["rus-Cyrl"],
    open_weights=True,
    revision="4036cab694767a299f2b9e6492909664d9414229",
    release_date="2020-03-04",
    n_parameters=1280_000_000,
    memory_usage_mb=4883,
    embed_dim=768,
    license="not specified",
    max_tokens=512,
    reference="https://huggingface.co/DeepPavlov/rubert-base-cased",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    adapted_from="google/bert_uncased_L-12_H-768_A-12",
    training_datasets={
        # wikipedia
        "WikipediaRetrievalMultilingual",
        "WikipediaRerankingMultilingual",
    },
    citation="""@misc{kuratov2019adaptationdeepbidirectionalmultilingual,
      title={Adaptation of Deep Bidirectional Multilingual Transformers for Russian Language},
      author={Yuri Kuratov and Mikhail Arkhipov},
      year={2019},
      eprint={1905.07213},
      archivePrefix={arXiv},
      primaryClass={cs.CL},
      url={https://arxiv.org/abs/1905.07213},
    }""",
)

distilrubert_small_cased_conversational = ModelMeta(
    loader=sentence_transformers_loader,
    name="DeepPavlov/distilrubert-small-cased-conversational",
    languages=["rus-Cyrl"],
    open_weights=True,
    revision="e348066b4a7279b97138038299bddc6580a9169a",
    release_date="2022-06-28",
    n_parameters=107_000_000,
    memory_usage_mb=408,
    embed_dim=768,
    license="not specified",
    max_tokens=512,
    reference="https://huggingface.co/DeepPavlov/distilrubert-small-cased-conversational",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    adapted_from="DeepPavlov/distilrubert-base-cased-conversational",
    training_datasets=set(
        # OpenSubtitles[1], Dirty, Pikabu, and a Social Media segment of Taiga corpus
    ),
    citation="""@misc{https://doi.org/10.48550/arxiv.2205.02340,
      doi = {10.48550/ARXIV.2205.02340},
      url = {https://arxiv.org/abs/2205.02340},
      author = {Kolesnikova, Alina and Kuratov, Yuri and Konovalov, Vasily and Burtsev, Mikhail},
      keywords = {Computation and Language (cs.CL), Machine Learning (cs.LG), FOS: Computer and information sciences, FOS: Computer and information sciences},
      title = {Knowledge Distillation of Russian Language Models with Reduction of Vocabulary},
      publisher = {arXiv},
      year = {2022},
      copyright = {arXiv.org perpetual, non-exclusive license}
    }""",
)

rubert_base_cased_sentence = ModelMeta(
    loader=sentence_transformers_loader,
    name="DeepPavlov/rubert-base-cased-sentence",
    languages=["rus-Cyrl"],
    open_weights=True,
    revision="78b5122d6365337dd4114281b0d08cd1edbb3bc8",
    release_date="2020-03-04",
    n_parameters=107_000_000,
    memory_usage_mb=408,
    embed_dim=768,
    license="not specified",
    max_tokens=512,
    reference="https://huggingface.co/DeepPavlov/rubert-base-cased-sentence",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets=set(
        # "SNLI",
        "XNLI"
    ),
)

labse_en_ru = ModelMeta(
    loader=sentence_transformers_loader,
    name="cointegrated/LaBSE-en-ru",
    languages=["rus-Cyrl"],
    open_weights=True,
    revision="cf0714e606d4af551e14ad69a7929cd6b0da7f7e",
    release_date="2021-06-10",
    n_parameters=129_000_000,
    memory_usage_mb=492,
    embed_dim=768,
    license="not specified",
    max_tokens=512,
    reference="https://huggingface.co/cointegrated/LaBSE-en-ru",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch"],
    use_instructions=False,
    public_training_code="https://colab.research.google.com/drive/1dnPRn0-ugj3vZgSpyCC9sgslM2SuSfHy?usp=sharing",
    public_training_data=None,
    training_datasets=set(
        # https://translate.yandex.ru/corpus
    ),
    adapted_from="sentence-transformers/LaBSE",
)

turbo_models_datasets = set(
    # Not MTEB: {"IlyaGusev/gazeta", "zloelias/lenta-ru"},
)
rubert_tiny_turbo = ModelMeta(
    loader=sentence_transformers_loader,
    name="sergeyzh/rubert-tiny-turbo",
    languages=["rus-Cyrl"],
    open_weights=True,
    revision="8ce0cf757446ce9bb2d5f5a4ac8103c7a1049054",
    release_date="2024-06-21",
    n_parameters=29_200_000,
    memory_usage_mb=111,
    embed_dim=312,
    license="mit",
    max_tokens=2048,
    reference="https://huggingface.co/sergeyzh/rubert-tiny-turbo",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets=turbo_models_datasets,
    adapted_from="cointegrated/rubert-tiny2",
)

rubert_mini_frida = ModelMeta(
    loader=sentence_transformers_loader,
    name="sergeyzh/rubert-mini-frida",
    languages=["rus-Cyrl"],
    open_weights=True,
    revision="19b279b78afd945b5ccae78f63e284909814adc2",
    release_date="2025-03-02",
    n_parameters=32_300_000,
    memory_usage_mb=123,
    embed_dim=312,
    license="mit",
    max_tokens=2048,
    reference="https://huggingface.co/sergeyzh/rubert-mini-frida",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch"],
    use_instructions=True,
    public_training_code=None,
    public_training_data=None,
    training_datasets=set(
        # https://huggingface.co/datasets/IlyaGusev/gazeta
        # https://huggingface.co/datasets/zloelias/lenta-ru
        # https://huggingface.co/datasets/HuggingFaceFW/fineweb-2
        # https://huggingface.co/datasets/HuggingFaceFW/fineweb
    ),
    adapted_from="sergeyzh/rubert-mini-sts",
)

labse_ru_turbo = ModelMeta(
    loader=sentence_transformers_loader,
    name="sergeyzh/LaBSE-ru-turbo",
    languages=["rus-Cyrl"],
    open_weights=True,
    revision="1940b046c6b5e125df11722b899130329d0a46da",
    release_date="2024-06-27",
    n_parameters=129_000_000,
    memory_usage_mb=490,
    embed_dim=768,
    license="mit",
    max_tokens=512,
    reference="https://huggingface.co/sergeyzh/LaBSE-ru-turbo",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch"],
    use_instructions=False,
    training_datasets=turbo_models_datasets,
    public_training_code=None,
    adapted_from="cointegrated/LaBSE-en-ru",
    public_training_data=None,
)

rosberta_prompts = {
    # Default
    "Classification": "classification: ",
    "MultilabelClassification": "classification: ",
    "Clustering": "clustering: ",
    "PairClassification": "classification: ",
    "Reranking": "classification: ",
    f"Reranking-{PromptType.query.value}": "search_query: ",
    f"Reranking-{PromptType.document.value}": "search_document: ",
    "STS": "classification: ",
    "Summarization": "classification: ",
    PromptType.query.value: "search_query: ",
    PromptType.document.value: "search_document: ",
    # Override some prompts for ruMTEB tasks
    "HeadlineClassification": "clustering: ",
    "InappropriatenessClassification": "clustering: ",
    "MassiveScenarioClassification": "clustering: ",
    "RuSciBenchGRNTIClassification": "clustering: ",
    "RuSciBenchOECDClassification": "clustering: ",
    "SensitiveTopicsClassification": "clustering: ",
    "STS22": "clustering: ",
}

rosberta_ru_en = ModelMeta(
    loader=sentence_transformers_loader,
    loader_kwargs=dict(
        model_prompts=rosberta_prompts,
    ),
    name="ai-forever/ru-en-RoSBERTa",
    languages=["rus-Cyrl"],
    open_weights=True,
    revision="89fb1651989adbb1cfcfdedafd7d102951ad0555",
    release_date="2024-07-29",
    use_instructions=True,
    reference="https://huggingface.co/ai-forever/ru-en-RoSBERTa",
    n_parameters=404_000_000,
    memory_usage_mb=1540,
    max_tokens=512,
    embed_dim=1024,
    license="mit",
    similarity_fn_name=ScoringFunction.COSINE,
    adapted_from="ai-forever/ruRoberta-large",
    training_datasets={
        # https://huggingface.co/ai-forever/ruRoberta-large
        # https://huggingface.co/datasets/IlyaGusev/yandex_q_full
        # https://huggingface.co/datasets/IlyaGusev/pikabu
        # https://huggingface.co/datasets/IlyaGusev/ru_stackoverflow
        # https://huggingface.co/datasets/IlyaGusev/habr
        # https://huggingface.co/datasets/its5Q/habr_qna
        # NewsCommentary
        # MultiParaCrawl
        "XNLI",
        "XNLIV2",
        "LanguageClassification",  # XNLI
        "MIRACLReranking",
        "MIRACLRetrieval",
        "MIRACLRetrievalHardNegatives",
        "MrTidyRetrieval",
    },
    public_training_data=None,
    public_training_code=None,
    framework=["Sentence Transformers", "PyTorch"],
    citation="""@misc{snegirev2024russianfocusedembeddersexplorationrumteb,
      title={The Russian-focused embedders' exploration: ruMTEB benchmark and Russian embedding model design},
      author={Artem Snegirev and Maria Tikhonova and Anna Maksimova and Alena Fenogenova and Alexander Abramov},
      year={2024},
      eprint={2408.12503},
      archivePrefix={arXiv},
      primaryClass={cs.CL},
      url={https://arxiv.org/abs/2408.12503},
    }
    """,
)

frida_prompts = {
    # Default
    "Classification": "categorize: ",
    "MultilabelClassification": "categorize: ",
    "Clustering": "categorize_topic: ",
    "PairClassification": "paraphrase: ",
    "Reranking": "paraphrase: ",
    f"Reranking-{PromptType.query.value}": "search_query: ",
    f"Reranking-{PromptType.document.value}": "search_document: ",
    "STS": "paraphrase: ",
    "Summarization": "categorize: ",
    PromptType.query.value: "search_query: ",
    PromptType.document.value: "search_document: ",
    # Override some prompts for ruMTEB tasks
    "CEDRClassification": "categorize_sentiment: ",
    "GeoreviewClassification": "categorize_sentiment: ",
    "HeadlineClassification": "categorize_topic: ",
    "InappropriatenessClassification": "categorize_topic: ",
    "KinopoiskClassification": "categorize_sentiment: ",
    "MassiveIntentClassification": "paraphrase: ",
    "MassiveScenarioClassification": "paraphrase: ",
    "RuReviewsClassification": "categorize_sentiment: ",
    "RuSciBenchGRNTIClassification": "categorize_topic: ",
    "RuSciBenchOECDClassification": "categorize_topic: ",
    "SensitiveTopicsClassification": "categorize_topic: ",
    "TERRa": "categorize_entailment: ",
    "RiaNewsRetrieval": "categorize: ",
}

frida_training_datasets = {
    # Fine-tune sets
    # Retrieval
    "MIRACLReranking",
    "MIRACLRetrieval",
    "MIRACLRetrievalHardNegatives",
    "MrTidyRetrieval",
    "MSMARCO",
    "MSMARCOHardNegatives",
    "NanoMSMARCORetrieval",
    "NQ",
    "NQHardNegatives",
    "NanoNQRetrieval",
    # STS
    "STS12",
    "STS22",
    "STSBenchmark",
    "STSBenchmarkMultilingualSTS",  # translation not trained on
    "RUParaPhraserSTS",
    # Classification & Clustering
    "AmazonCounterfactualClassification",
    "AmazonPolarityClassification",
    "AmazonReviewsClassification",
    "ArxivClusteringP2P.v2",
    "ArxivClusteringP2P",
    "Banking77Classification",
    "BiorxivClusteringP2P.v2",
    "BiorxivClusteringP2P",
    "CEDRClassification",
    "DBpediaClassification",
    "EmotionClassification",
    "FinancialPhrasebankClassification",
    "FrenkEnClassification",
    "GeoreviewClassification",
    "GeoreviewClusteringP2P",
    "HeadlineClassification",
    "ImdbClassification",
    "InappropriatenessClassification",
    "KinopoiskClassification",
    "MasakhaNEWSClassification",
    "MassiveIntentClassification",
    "MassiveScenarioClassification",
    "MedrxivClusteringP2P.v2",
    "MedrxivClusteringP2P",
    "MTOPDomainClassification",
    "MTOPIntentClassification",
    "MultiHateClassification",
    "MultilingualSentimentClassification",
    "NewsClassification",
    "NusaX-senti",
    "PoemSentimentClassification",
    "RuReviewsClassification",
    "RuSciBenchGRNTIClassification",
    "RuSciBenchOECDClassification",
    "SensitiveTopicsClassification",
    "SIB200Classification",
    "ToxicChatClassification",
    "ToxicConversationsClassification",
    "TweetSentimentClassification",
    "TweetSentimentExtractionClassification",
    "TweetTopicSingleClassification",
    "YahooAnswersTopicsClassification",
    "YelpReviewFullClassification",
    # Pre-train sets (not mentioned above)
    # https://github.com/nomic-ai/contrastors/blob/5f7b461e5a13b5636692d1c9f1141b27232fe966/src/contrastors/configs/data/contrastive_pretrain.yaml
    # reddit_title_body
    "RedditClustering",
    "RedditClusteringP2P",
    "RedditClustering.v2",
    # codesearch
    "CodeSearchNetCCRetrieval",
    "COIRCodeSearchNetRetrieval",
    # stackexchange_body_body
    "StackExchangeClustering.v2",
    "StackExchangeClusteringP2P.v2",
    # wikipedia
    "WikipediaRetrievalMultilingual",
    "WikipediaRerankingMultilingual",
    # quora
    "QuoraRetrieval",
    "NanoQuoraRetrieval",
    "Quora-NL",  # translation not trained on
}

frida = ModelMeta(
    loader=sentence_transformers_loader,
    loader_kwargs=dict(
        model_prompts=frida_prompts,
    ),
    name="ai-forever/FRIDA",
    languages=["rus-Cyrl"],
    open_weights=True,
    revision="7292217af9a9e6dbf07048f76b434ad1e2aa8b76",
    release_date="2024-12-29",
    use_instructions=True,
    reference="https://huggingface.co/ai-forever/FRIDA",
    n_parameters=823_000_000,
    memory_usage_mb=3141,
    max_tokens=512,
    embed_dim=1536,
    license="mit",
    similarity_fn_name=ScoringFunction.COSINE,
    adapted_from="ai-forever/FRED-T5-1.7B",
    training_datasets=frida_training_datasets,
    public_training_data=None,
    public_training_code=None,
    framework=["Sentence Transformers", "PyTorch"],
)

giga_embeddings = ModelMeta(
    loader=InstructSentenceTransformerModel,
    loader_kwargs=dict(
        instruction_template="Instruct: {instruction}\nQuery: ",
        max_seq_length=4096,
        trust_remote_code=True,
        apply_instruction_to_passages=True,
        prompts_dict=GIGA_task_prompts,
        model_kwargs={
            "torch_dtype": torch.bfloat16,
        },
    ),
    name="ai-sage/Giga-Embeddings-instruct",
    languages=["eng-Latn", "rus-Cyrl"],
    open_weights=True,
    revision="0ad5b29bfecd806cecc9d66b927d828a736594dc",
    release_date="2025-09-23",
    n_parameters=3_227_176_961,
    memory_usage_mb=12865,
    embed_dim=2048,
    license="mit",
    max_tokens=4096,
    reference="https://huggingface.co/ai-sage/Giga-Embeddings-instruct",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch"],
    use_instructions=True,
    public_training_code=None,
    public_training_data=None,
    training_datasets=None,
)

berta_training_datasets = (
    set(
        # https://huggingface.co/datasets/IlyaGusev/gazeta
        # https://huggingface.co/datasets/zloelias/lenta-ru
        # https://huggingface.co/datasets/HuggingFaceFW/fineweb-2
        # https://huggingface.co/datasets/HuggingFaceFW/fineweb
    )
    | frida_training_datasets
)  # distilled from FRIDA

berta = ModelMeta(
    loader=sentence_transformers_loader,
    name="sergeyzh/BERTA",
    languages=["rus-Cyrl"],
    open_weights=True,
    revision="914c8c8aed14042ed890fc2c662d5e9e66b2faa7",
    release_date="2025-03-10",
    n_parameters=128_000_000,
    memory_usage_mb=489,
    embed_dim=768,
    license="mit",
    max_tokens=512,
    reference="https://huggingface.co/sergeyzh/BERTA",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch"],
    use_instructions=True,
    training_datasets=berta_training_datasets,
    public_training_code=None,
    adapted_from="sergeyzh/LaBSE-ru-turbo",
    public_training_data=None,
)


user2_training_data = (
    set(
        # deepvk/cultura_ru_edu
        # AllNLI
        # nyuuzyou/fishkinet-posts
        # IlyaGusev/gazeta
        # its5Q/habr_qna
        # zloelias/lenta-ru
        # unicamp-dl/mmarco
        # deepvk/ru-HNP
        # deepvk/ru-WANLI
        # wikimedia/wikipedia
        # CarlBrendt/Summ_Dialog_News
        # RussianNLP/wikiomnia
        # its5Q/yandex-q
        # "mC4" ru
        # "CC-News" ru
        # MultiLongDocRetrieval
    )
    | nomic_training_data
    | bge_m3_training_data
)

user2_prompts = {
    # Override some prompts for ruMTEB tasks
    "HeadlineClassification": "search_query: ",
    "RuSciBenchGRNTIClassification": "clustering: ",
    "RuSciBenchOECDClassification": "clustering: ",
    "GeoreviewClusteringP2P": "search_query: ",
    "SensitiveTopicsClassification": "search_query: ",
    "STS22": "search_document: ",
    "InappropriatenessClassification": "classification: ",
    "CEDRClassification": "classification: ",
    # Default
    "Classification": "classification: ",
    "MultilabelClassification": "classification: ",
    "Clustering": "clustering: ",
    "PairClassification": "classification: ",
    "Reranking": "classification: ",
    f"Reranking-{PromptType.query.value}": "search_query: ",
    f"Reranking-{PromptType.document.value}": "search_document: ",
    "STS": "classification: ",
    "Summarization": "clustering: ",
    PromptType.query.value: "search_query: ",
    PromptType.document.value: "search_document: ",
}
user2_small = ModelMeta(
    loader=sentence_transformers_loader,
    loader_kwargs=dict(
        model_prompts=user2_prompts,
    ),
    name="deepvk/USER2-small",
    languages=["rus-Cyrl"],
    open_weights=True,
    revision="23f65b34cf7632032061f5cc66c14714e6d4cee4",
    release_date="2025-04-19",
    use_instructions=True,
    reference="https://huggingface.co/collections/deepvk/user2-6802650d7210f222ec60e05f",
    n_parameters=34_400_000,
    memory_usage_mb=131,
    max_tokens=8192,
    embed_dim=384,
    license="apache-2.0",
    similarity_fn_name="cosine",
    adapted_from="deepvk/RuModernBERT-small",
    training_datasets=user2_training_data,
    public_training_data=None,
    public_training_code="https://github.com/BlessedTatonka/some_code/tree/2899f27d51efdf4217fc6453799ff197e9792f1e",
    framework=["Sentence Transformers", "PyTorch"],
)

user2_base = ModelMeta(
    loader=sentence_transformers_loader,
    loader_kwargs=dict(
        model_prompts=user2_prompts,
    ),
    name="deepvk/USER2-base",
    languages=["rus-Cyrl"],
    open_weights=True,
    revision="0942cf96909b6d52e61f79a01e2d30c7be640b27",
    release_date="2025-04-19",
    use_instructions=True,
    reference="https://huggingface.co/collections/deepvk/user2-6802650d7210f222ec60e05f",
    n_parameters=149_000_000,
    memory_usage_mb=568,
    max_tokens=8192,
    embed_dim=768,
    license="apache-2.0",
    similarity_fn_name="cosine",
    adapted_from="deepvk/RuModernBERT-base",
    training_datasets=user2_training_data,
    public_training_data=None,
    public_training_code="https://github.com/BlessedTatonka/some_code/tree/2899f27d51efdf4217fc6453799ff197e9792f1e",
    framework=["Sentence Transformers", "PyTorch"],
)



from abc import ABC, abstractmethod
from typing import Any

from mteb.abstasks.abstask import _set_seed
from mteb.models import EncoderProtocol


class Evaluator(ABC):
    """Base class for all evaluators

    Extend this class and implement __call__ for custom evaluators.
    """

    def __init__(self, seed: int = 42, **kwargs: Any) -> None:
        self.seed = seed
        self.rng_state, self.np_rng = _set_seed(seed)

    @abstractmethod
    def __call__(
        self, model: EncoderProtocol, *, encode_kwargs: dict[str, Any]
    ) -> dict[str, float]:
        """This is called during training to evaluate the model.

        It returns scores.

        Args:
            model: the model to evaluate
            encode_kwargs: kwargs to pass to the model's encode method
        """
        pass



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata

_LANGUAGES = {
    "gu": ["guj-Gujr"],
    "kn": ["kan-Knda"],
    "mal": ["mal-Mlym"],
    "mr": ["mar-Deva"],
    "tel": ["tel-Telu"],
    "ori": ["ori-Orya"],
    "pa": ["pan-Guru"],
    "ta": ["tam-Taml"],
}


class IndicNLPNewsClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="IndicNLPNewsClassification",
        dataset={
            "path": "Sakshamrzt/IndicNLP-Multilingual",
            "revision": "3f23bd4a622a462adfb6989419cfadf7dc778f25",
        },
        description="A News classification dataset in multiple Indian regional languages.",
        reference="https://github.com/AI4Bharat/indicnlp_corpus#indicnlp-news-article-classification-dataset",
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=_LANGUAGES,
        main_score="accuracy",
        date=("2020-09-01", "2022-04-09"),
        domains=["News", "Written"],
        dialect=[],
        task_subtypes=["Topic classification"],
        license="cc-by-nc-4.0",
        annotations_creators="expert-annotated",
        sample_creation="found",
        bibtex_citation=r"""
@article{kunchukuttan2020indicnlpcorpus,
  author = {Anoop Kunchukuttan and Divyanshu Kakwani and Satish Golla and Gokul N.C. and Avik Bhattacharyya and Mitesh M. Khapra and Pratyush Kumar},
  journal = {arXiv preprint arXiv:2005.00085},
  title = {AI4Bharat-IndicNLP Corpus: Monolingual Corpora and Word Embeddings for Indic Languages},
  year = {2020},
}
""",
    )

    def dataset_transform(self):
        for lang in self.hf_subsets:
            self.dataset[lang] = self.dataset[lang].rename_columns(
                {"news": "text", "class": "label"}
            )
            if lang == "pa":
                self.dataset[lang] = self.dataset[lang].remove_columns("headline")
            if self.dataset[lang]["test"].num_rows > 2048:
                self.dataset[lang] = self.stratified_subsampling(
                    self.dataset[lang],
                    seed=self.seed,
                    splits=["test"],
                )



from mteb.abstasks import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class CIFAR10Classification(AbsTaskClassification):
    input_column_name: str = "img"
    samples_per_label: int = 16
    n_experiments: int = 5

    metadata = TaskMetadata(
        name="CIFAR10",
        description="Classifying images from 10 classes.",
        reference="https://huggingface.co/datasets/uoft-cs/cifar10",
        dataset={
            "path": "uoft-cs/cifar10",
            "revision": "0b2714987fa478483af9968de7c934580d0bb9a2",
        },
        type="ImageClassification",
        category="i2c",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=(
            "2008-01-01",
            "2009-01-01",
        ),  # Estimated range for the collection of reviews
        domains=["Web"],
        task_subtypes=["Object recognition"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        modalities=["image"],
        sample_creation="created",
        bibtex_citation=r"""
@techreport{Krizhevsky09learningmultiple,
  author = {Alex Krizhevsky},
  institution = {},
  title = {Learning multiple layers of features from tiny images},
  year = {2009},
}
""",
    )


class CIFAR100Classification(AbsTaskClassification):
    input_column_name: str = "img"
    label_column_name: str = "fine_label"
    samples_per_label: int = 16
    n_experiments: int = 5

    metadata = TaskMetadata(
        name="CIFAR100",
        description="Classifying images from 100 classes.",
        reference="https://huggingface.co/datasets/uoft-cs/cifar100",
        dataset={
            "path": "uoft-cs/cifar100",
            "revision": "aadb3af77e9048adbea6b47c21a81e47dd092ae5",
        },
        type="ImageClassification",
        category="i2t",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=(
            "2008-01-01",
            "2009-01-01",
        ),  # Estimated range for the collection of reviews
        domains=["Web"],
        task_subtypes=["Object recognition"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        modalities=["image"],
        sample_creation="created",
        bibtex_citation=r"""
@techreport{Krizhevsky09learningmultiple,
  author = {Alex Krizhevsky},
  institution = {},
  title = {Learning multiple layers of features from tiny images},
  year = {2009},
}
""",
    )



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class FeedbackQARetrieval(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="FeedbackQARetrieval",
        description="Using Interactive Feedback to Improve the Accuracy and Explainability of Question Answering Systems Post-Deployment",
        reference="https://arxiv.org/abs/2204.03025",
        dataset={
            "path": "mteb/FeedbackQARetrieval",
            "revision": "8a17afc1b3e112c65fd6938b75692846afda97f8",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="precision_at_1",
        date=("2020-01-01", "2022-04-01"),
        domains=["Web", "Government", "Medical", "Written"],
        task_subtypes=["Question answering"],
        license="apache-2.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="created",
        bibtex_citation=r"""
@inproceedings{li-etal-2022-using,
  address = {Dublin, Ireland},
  author = {Li, Zichao  and
Sharma, Prakhar  and
Lu, Xing Han  and
Cheung, Jackie  and
Reddy, Siva},
  booktitle = {Findings of the Association for Computational Linguistics: ACL 2022},
  doi = {10.18653/v1/2022.findings-acl.75},
  editor = {Muresan, Smaranda  and
Nakov, Preslav  and
Villavicencio, Aline},
  month = may,
  pages = {926--937},
  publisher = {Association for Computational Linguistics},
  title = {Using Interactive Feedback to Improve the Accuracy and Explainability of Question Answering Systems Post-Deployment},
  url = {https://aclanthology.org/2022.findings-acl.75},
  year = {2022},
}
""",
    )



import itertools
import logging
import random
from collections import defaultdict
from pathlib import Path
from typing import Any

import numpy as np
from datasets import Dataset, DatasetDict
from sklearn.cluster import MiniBatchKMeans
from sklearn.metrics.cluster import v_measure_score

from mteb._create_dataloaders import create_dataloader
from mteb.models import EncoderProtocol
from mteb.types import HFSubset, ScoresDict
from mteb.types.statistics import (
    ImageStatistics,
    LabelStatistics,
    SplitDescriptiveStatistics,
    TextStatistics,
)

from ._statistics_calculation import (
    calculate_image_statistics,
    calculate_label_statistics,
    calculate_text_statistics,
)
from .abstask import AbsTask

logger = logging.getLogger(__name__)


MultilingualDataset = dict[HFSubset, DatasetDict]


def _evaluate_clustering_bootstrapped(
    embeddings: np.ndarray,
    labels: list[list[str]],
    n_clusters: int,
    cluster_size: int,
    kmean_batch_size: int,
    max_depth: int | None,
    rng_state: random.Random,
    seed: int,
) -> tuple[dict[str, list[float]], dict[str, list[list[int]]]]:
    """Bootstrapped evaluation of clustering performance using V-measure.

    The bootstrapping is done by sampling N samples from the corpus and clustering them. It is done without replacement to get a diverse set of
    samples.

    Returns:
        A tuple containing:
        - A dictionary where keys are level names (e.g., "Level 0", "Level 1", etc.) and values are lists of V-measure scores for each clustering experiment at that level.
        - A dictionary where keys are level names and values are lists of cluster assignments for each clustering experiment at that level.
    """
    v_measures = defaultdict(list)
    cluster_assignments = defaultdict(list)
    if max_depth is not None:
        max_depth = min(max_depth, max(map(len, labels)))
    else:
        max_depth = max(map(len, labels))
    # Evaluate on each level til max depth
    for i_level in range(max_depth):
        level_labels = []
        # Assign -1 to gold label if the level is not there
        for label in labels:
            if len(label) > i_level:
                level_labels.append(label[i_level])
            else:
                level_labels.append(-1)
        level_labels = np.array(level_labels)
        valid_idx = np.array(
            [level_label != -1 for level_label in level_labels]
        )  # Could be level_labels != -1 but fails with FutureWarning: elementwise comparison failed
        level_labels = level_labels[valid_idx]
        level_embeddings = embeddings[valid_idx]
        clustering_model = MiniBatchKMeans(
            n_clusters=np.unique(level_labels).size,
            batch_size=kmean_batch_size,
            init="k-means++",
            n_init=1,  # default when kmeans++ is used
            random_state=seed,
        )
        for _ in range(n_clusters):
            # sample N samples from the corpus with replacement
            n_embeddings = len(level_embeddings)
            cluster_indices = rng_state.choices(range(n_embeddings), k=cluster_size)

            _embeddings = level_embeddings[cluster_indices]
            _labels = level_labels[cluster_indices]
            cluster_assignment = clustering_model.fit_predict(_embeddings)
            v_measure = v_measure_score(_labels, cluster_assignment)
            v_measures[f"Level {i_level}"].append(v_measure)
            cluster_assignments[f"Level {i_level}"].append(cluster_assignment.tolist())

    return v_measures, cluster_assignments


class ClusteringFastDescriptiveStatistics(SplitDescriptiveStatistics):
    """Descriptive statistics for ClusteringFast

    Attributes:
        num_samples: number of samples in the dataset.

        text_statistics: Statistics for text
        image_statistics: Statistics for images
        labels_statistics: Statistics for labels
    """

    num_samples: int

    text_statistics: TextStatistics | None
    image_statistics: ImageStatistics | None
    labels_statistics: LabelStatistics


class AbsTaskClustering(AbsTask):
    """Abstract class for Clustering tasks.

    This class embeds the corpus sentences then samples N samples from the corpus and clusters them.
    The similarity then is calculated using the V-measure metric, which is invariant to the permutation of the labels.
    This approach is then repeated K times.

    There are two ways to specify how a dataset is downsampled `max_document_to_embed` and `max_fraction_of_documents_to_embed`.
    If both parameters are set to None, no downsampling is done in self._evaluate_subset().
    Only one of these two parameters can be not None at the same time.

    If the clustering is hierarchical, and more than one label is specified in order for each observation,
    V-measures are calculated in the outlined way on each of the levels separately.

    Attributes:
        dataset: A HuggingFace Dataset containing the data for the clustering task. Must contain the following columns `sentences` that contains inputs (texts or images) and labels columns.
        max_fraction_of_documents_to_embed: Fraction of documents to embed for clustering.
        max_document_to_embed: Maximum number of documents to embed for clustering.
        max_documents_per_cluster: Number of documents to sample for each clustering experiment.
        n_clusters: Number of clustering experiments to run.
        k_mean_batch_size: Batch size to use for k-means clustering.
        max_depth: Maximum depth to evaluate clustering. If None, evaluates all levels.
        input_column_name: Name of the column containing the input sentences or data points.
        label_column_name: Name of the column containing the true cluster labels.
        abstask_prompt: Prompt to use for the task for instruction model if not prompt is provided in TaskMetadata.prompt.
    """

    max_fraction_of_documents_to_embed: float | None = 0.04
    max_document_to_embed: int | None = None
    max_documents_per_cluster: int = 16_384
    n_clusters: int = 10
    k_mean_batch_size: int = 512
    max_depth = None
    abstask_prompt = "Identify categories in user passages."
    input_column_name: str = "sentences"
    label_column_name: str = "labels"

    def _evaluate_subset(
        self,
        model: EncoderProtocol,
        data_split: Dataset,
        *,
        encode_kwargs: dict[str, Any],
        hf_split: str,
        hf_subset: str,
        prediction_folder: Path | None = None,
        **kwargs: Any,
    ) -> ScoresDict:
        if (
            self.max_document_to_embed is not None
            and self.max_fraction_of_documents_to_embed is not None
        ):
            raise Exception(
                "Both max_document_to_embed and max_fraction_of_documents_to_embed are set. Please only set one."
            )

        logger.info("Running clustering - Preparing data...")
        if (
            self.max_document_to_embed is None
            and self.max_fraction_of_documents_to_embed is None
        ):
            downsampled_dataset = data_split
        else:
            if self.max_fraction_of_documents_to_embed is not None:
                max_documents_to_embed = int(
                    self.max_fraction_of_documents_to_embed * len(data_split)
                )
            else:
                max_documents_to_embed = self.max_document_to_embed

            max_documents_to_embed = min(len(data_split), max_documents_to_embed)  # type: ignore
            example_indices = self.rng_state.sample(
                range(len(data_split)), k=max_documents_to_embed
            )
            downsampled_dataset = data_split.select(example_indices)  # type: ignore

        downsampled_dataset = downsampled_dataset.select_columns(
            [self.input_column_name, self.label_column_name]
        )

        logger.info("Running clustering - Encoding samples...")
        embeddings = model.encode(
            create_dataloader(
                downsampled_dataset,
                self.metadata,
                input_column=self.input_column_name,
                **encode_kwargs,
            ),
            task_metadata=self.metadata,
            hf_subset=hf_subset,
            hf_split=hf_split,
            **encode_kwargs,
        )

        logger.info("Running clustering - Evaluating clustering...")
        labels = []
        for label in downsampled_dataset[self.label_column_name]:
            if not isinstance(label, list):
                label = [label]
            labels.append(label)

        all_v_scores, all_assignments = _evaluate_clustering_bootstrapped(
            embeddings,
            labels,
            n_clusters=self.n_clusters,
            cluster_size=self.max_documents_per_cluster,
            kmean_batch_size=self.k_mean_batch_size,
            max_depth=self.max_depth,
            rng_state=self.rng_state,
            seed=self.seed,
        )

        if prediction_folder:
            self._save_task_predictions(
                all_assignments,
                model,
                prediction_folder,
                hf_subset=hf_subset,
                hf_split=hf_split,
            )

        v_measures = list(itertools.chain.from_iterable(all_v_scores.values()))

        logger.info("Running clustering - Finished.")
        mean_v_measure = np.mean(v_measures)
        v_std = np.std(v_measures)
        return {
            "v_measures": all_v_scores,
            "v_measure": float(mean_v_measure),
            "v_measure_std": v_std,
        }

    def _calculate_descriptive_statistics_from_split(
        self, split: str, hf_subset: str | None = None, compute_overall: bool = False
    ) -> ClusteringFastDescriptiveStatistics:
        if hf_subset:
            inputs = self.dataset[hf_subset][split][self.input_column_name]
            labels = self.dataset[hf_subset][split][self.label_column_name]
        elif compute_overall:
            inputs = []
            labels = []
            for hf_subset in self.metadata.eval_langs:
                inputs.extend(self.dataset[hf_subset][split][self.input_column_name])
                labels.extend(self.dataset[hf_subset][split][self.label_column_name])
        else:
            inputs = self.dataset[split][self.input_column_name]
            labels = self.dataset[split][self.label_column_name]

        if isinstance(inputs[0], list):
            inputs = [item for sublist in inputs for item in sublist]
        if isinstance(labels[0], list):
            labels = [item for sublist in labels for item in sublist]

        text_statistics, image_statistics = None, None
        if "image" in self.metadata.modalities:
            image_statistics = calculate_image_statistics(inputs)

        if "text" in self.metadata.modalities:
            text_statistics = calculate_text_statistics(inputs)

        label_statistics = calculate_label_statistics(labels)

        return ClusteringFastDescriptiveStatistics(
            num_samples=len(inputs),
            text_statistics=text_statistics,
            image_statistics=image_statistics,
            labels_statistics=label_statistics,
        )

    def _push_dataset_to_hub(self, repo_name: str) -> None:
        self._upload_dataset_to_hub(
            repo_name, [self.input_column_name, self.label_column_name]
        )


def _convert_to_fast(
    dataset: DatasetDict,
    input_column_name: str,
    label_column_name: str,
    seed: int,
    max_size: int = 100_000,
) -> DatasetDict:
    """Converts a clustering dataset to a fast version.

    This concat the cluster into two columns, sentences and labels. It additionally downsamples the dataset to max_size.

    Args:
        dataset: A DatasetDict containing the data for the clustering task. Must contain the following columns
        input_column_name: Name of the column containing the input sentences or data points.
        label_column_name: Name of the column containing the true cluster labels.
        seed: Random seed for downsampling.
        max_size: Maximum number of samples in the returned dataset.

    Returns:
        A downsampled DatasetDict with two columns, sentences and labels.
    """
    rng_state = random.Random(seed)

    ds = {}
    for split in dataset:
        sent_set = set()
        labels = []
        sentences = []
        n_clusters = len(dataset[split])
        all_labels_set = set(
            itertools.chain.from_iterable(dataset[split][label_column_name])
        )
        for i in range(n_clusters):
            lab = dataset[split][label_column_name][i]
            sents = dataset[split][input_column_name][i]

            # check that it is the same distribution
            row_label_set = set(lab)
            assert row_label_set.issubset(all_labels_set), (
                "The clusters are not sampled from the same distribution as they have different labels."
            )

            for l, s in zip(lab, sents):
                if s not in sent_set:
                    labels.append(l)
                    sentences.append(s)
                    sent_set.add(s)  # ensuring no duplicates

        ds[split] = Dataset.from_dict(
            {input_column_name: sentences, label_column_name: labels}
        )

        if len(ds[split]) > max_size:
            idxs = rng_state.sample(range(len(ds[split])), max_size)
            ds[split] = ds[split].select(idxs)

    return DatasetDict(ds)


def _check_label_distribution(
    ds: DatasetDict,
    label_column_name: str = "labels",
) -> None:
    """For older clustering dataset versions.

    Checks that all clusters are sampled from the same distribution by checking that the set of labels is the same across clusters.

    Args:
        ds: A DatasetDict containing the data for the clustering task. Must contain the following columns
        label_column_name: Name of the column containing the true cluster labels.
    """
    n_clusters = len(ds)
    if n_clusters > 50:
        return
    all_labels_set = set(itertools.chain.from_iterable(ds[label_column_name]))

    for i in range(n_clusters):
        lab = ds[label_column_name][i]

        # check that it is the same distribution
        row_label_set = set(lab)
        assert row_label_set.issubset(all_labels_set), (
            "The clusters are not sampled from the same distribution as they have different labels."
        )



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class DKHateClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="DKHateClassification",
        dataset={
            "path": "DDSC/dkhate",
            "revision": "59d12749a3c91a186063c7d729ec392fda94681c",
        },
        description="Danish Tweets annotated for Hate Speech either being Offensive or not",
        reference="https://aclanthology.org/2020.lrec-1.430/",
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["dan-Latn"],
        main_score="accuracy",
        date=("2018-01-01", "2018-12-31"),
        domains=["Social", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="cc-by-4.0",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{sigurbergsson-derczynski-2020-offensive,
  address = {Marseille, France},
  author = {Sigurbergsson, Gudbjartur Ingi  and
Derczynski, Leon},
  booktitle = {Proceedings of the Twelfth Language Resources and Evaluation Conference},
  editor = {Calzolari, Nicoletta  and
B{\'e}chet, Fr{\'e}d{\'e}ric  and
Blache, Philippe  and
Choukri, Khalid  and
Cieri, Christopher  and
Declerck, Thierry  and
Goggi, Sara  and
Isahara, Hitoshi  and
Maegaard, Bente  and
Mariani, Joseph  and
Mazo, H{\'e}l{\`e}ne  and
Moreno, Asuncion  and
Odijk, Jan  and
Piperidis, Stelios},
  isbn = {979-10-95546-34-4},
  language = {English},
  month = may,
  pages = {3498--3508},
  publisher = {European Language Resources Association},
  title = {Offensive Language and Hate Speech Detection for {D}anish},
  url = {https://aclanthology.org/2020.lrec-1.430},
  year = {2020},
}
""",
        prompt="Classify Danish tweets based on offensiveness (offensive, not offensive)",
        superseded_by="DKHateClassification.v2",
    )

    samples_per_label = 16

    def dataset_transform(self):
        # convert label to a 0/1 label
        labels = self.dataset["train"]["label"]  # type: ignore
        lab2idx = {lab: idx for idx, lab in enumerate(set(labels))}
        self.dataset = self.dataset.map(
            lambda x: {"label": lab2idx[x["label"]]}, remove_columns=["label"]
        )


class DKHateClassificationV2(AbsTaskClassification):
    metadata = TaskMetadata(
        name="DKHateClassification.v2",
        dataset={
            "path": "mteb/dk_hate",
            "revision": "0468ff11393992d8347cf4282fb706fe970608d4",
        },
        description="Danish Tweets annotated for Hate Speech either being Offensive or not This version corrects errors found in the original data. For details, see [pull request](https://github.com/embeddings-benchmark/mteb/pull/2900)",
        reference="https://aclanthology.org/2020.lrec-1.430/",
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["dan-Latn"],
        main_score="accuracy",
        date=("2018-01-01", "2018-12-31"),
        domains=["Social", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="cc-by-4.0",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{sigurbergsson-derczynski-2020-offensive,
  address = {Marseille, France},
  author = {Sigurbergsson, Gudbjartur Ingi  and
Derczynski, Leon},
  booktitle = {Proceedings of the Twelfth Language Resources and Evaluation Conference},
  editor = {Calzolari, Nicoletta  and
B{\'e}chet, Fr{\'e}d{\'e}ric  and
Blache, Philippe  and
Choukri, Khalid  and
Cieri, Christopher  and
Declerck, Thierry  and
Goggi, Sara  and
Isahara, Hitoshi  and
Maegaard, Bente  and
Mariani, Joseph  and
Mazo, H{\'e}l{\`e}ne  and
Moreno, Asuncion  and
Odijk, Jan  and
Piperidis, Stelios},
  isbn = {979-10-95546-34-4},
  language = {English},
  month = may,
  pages = {3498--3508},
  publisher = {European Language Resources Association},
  title = {Offensive Language and Hate Speech Detection for {D}anish},
  url = {https://aclanthology.org/2020.lrec-1.430},
  year = {2020},
}
""",
        prompt="Classify Danish tweets based on offensiveness (offensive, not offensive)",
        adapted_from=["DKHateClassification"],
    )

    samples_per_label = 16



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class FrenkHrClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="FrenkHrClassification",
        description="Croatian subset of the FRENK dataset",
        dataset={
            "path": "mteb/FrenkHrClassification",
            "revision": "eb5117a28f51d6a4e613590174c39b530e275e53",
        },
        reference="https://arxiv.org/abs/1906.02045",
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["hrv-Latn"],
        main_score="accuracy",
        date=("2021-05-28", "2021-05-28"),
        domains=["Social", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@misc{ljubešić2019frenk,
  archiveprefix = {arXiv},
  author = {Nikola Ljubešić and Darja Fišer and Tomaž Erjavec},
  eprint = {1906.02045},
  primaryclass = {cs.CL},
  title = {The FRENK Datasets of Socially Unacceptable Discourse in Slovene and English},
  url = {https://arxiv.org/abs/1906.02045},
  year = {2019},
}
""",
        superseded_by="FrenkHrClassification.v2",
    )


class FrenkHrClassificationV2(AbsTaskClassification):
    metadata = TaskMetadata(
        name="FrenkHrClassification.v2",
        description="Croatian subset of the FRENK dataset This version corrects errors found in the original data. For details, see [pull request](https://github.com/embeddings-benchmark/mteb/pull/2900)",
        dataset={
            "path": "mteb/frenk_hr",
            "revision": "09f90d0bee34d5e703caed26737166591a8f12b9",
        },
        reference="https://arxiv.org/abs/1906.02045",
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["hrv-Latn"],
        main_score="accuracy",
        date=("2021-05-28", "2021-05-28"),
        domains=["Social", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@misc{ljubešić2019frenk,
  archiveprefix = {arXiv},
  author = {Nikola Ljubešić and Darja Fišer and Tomaž Erjavec},
  eprint = {1906.02045},
  primaryclass = {cs.CL},
  title = {The FRENK Datasets of Socially Unacceptable Discourse in Slovene and English},
  url = {https://arxiv.org/abs/1906.02045},
  year = {2019},
}
""",
        adapted_from=["FrenkHrClassification"],
    )



from mteb.abstasks.sts import AbsTaskSTS
from mteb.abstasks.task_metadata import TaskMetadata


class SickrPLSTS(AbsTaskSTS):
    metadata = TaskMetadata(
        name="SICK-R-PL",
        dataset={
            "path": "PL-MTEB/sickr-pl-sts",
            "revision": "fd5c2441b7eeff8676768036142af4cfa42c1339",
        },
        description="Polish version of SICK dataset for textual relatedness.",
        reference="https://aclanthology.org/2020.lrec-1.207",
        type="STS",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["pol-Latn"],
        main_score="cosine_spearman",
        date=("2018-01-01", "2019-09-01"),  # rough estimate
        domains=["Web", "Written"],
        task_subtypes=["Textual Entailment"],
        license="cc-by-nc-sa-3.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="human-translated and localized",
        bibtex_citation=r"""
@inproceedings{dadas-etal-2020-evaluation,
  address = {Marseille, France},
  author = {Dadas, Slawomir  and
Perelkiewicz, Michal  and
Poswiata, Rafal},
  booktitle = {Proceedings of the Twelfth Language Resources and Evaluation Conference},
  editor = {Calzolari, Nicoletta  and
B{\'e}chet, Fr{\'e}d{\'e}ric  and
Blache, Philippe  and
Choukri, Khalid  and
Cieri, Christopher  and
Declerck, Thierry  and
Goggi, Sara  and
Isahara, Hitoshi  and
Maegaard, Bente  and
Mariani, Joseph  and
Mazo, Helene  and
Moreno, Asuncion  and
Odijk, Jan  and
Piperidis, Stelios},
  isbn = {979-10-95546-34-4},
  language = {English},
  month = may,
  pages = {1674--1680},
  publisher = {European Language Resources Association},
  title = {Evaluation of Sentence Representations in {P}olish},
  url = {https://aclanthology.org/2020.lrec-1.207},
  year = {2020},
}
""",
    )

    min_score = 1
    max_score = 5


class CdscrSTS(AbsTaskSTS):
    metadata = TaskMetadata(
        name="CDSC-R",
        dataset={
            "path": "PL-MTEB/cdscr-sts",
            "revision": "1cd6abbb00df7d14be3dbd76a7dcc64b3a79a7cd",
        },
        description="Compositional Distributional Semantics Corpus for textual relatedness.",
        reference="https://aclanthology.org/P17-1073.pdf",
        type="STS",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["pol-Latn"],
        main_score="cosine_spearman",
        date=("2016-01-01", "2017-04-01"),  # rough estimate
        domains=["Web", "Written"],
        task_subtypes=["Textual Entailment"],
        license="cc-by-nc-sa-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="human-translated and localized",
        bibtex_citation=r"""
@inproceedings{wroblewska-krasnowska-kieras-2017-polish,
  address = {Vancouver, Canada},
  author = {Wr{\'o}blewska, Alina  and
Krasnowska-Kiera{\'s}, Katarzyna},
  booktitle = {Proceedings of the 55th Annual Meeting of the Association for Computational Linguistics (Volume 1: Long Papers)},
  doi = {10.18653/v1/P17-1073},
  editor = {Barzilay, Regina  and
Kan, Min-Yen},
  month = jul,
  pages = {784--792},
  publisher = {Association for Computational Linguistics},
  title = {{P}olish evaluation dataset for compositional distributional semantics models},
  url = {https://aclanthology.org/P17-1073},
  year = {2017},
}
""",
    )

    min_score = 1
    max_score = 5



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class ItaCaseholdClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="ItaCaseholdClassification",
        dataset={
            "path": "itacasehold/itacasehold",
            "revision": "fafcfc4fee815f7017848e54b26c47ece8ff1626",
        },
        description="An Italian Dataset consisting of 1101 pairs of judgments and their official holdings between the years 2019 and 2022 from the archives of Italian Administrative Justice categorized with 64 subjects.",
        reference="https://doi.org/10.1145/3594536.3595177",
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["ita-Latn"],
        main_score="accuracy",
        date=("2019-01-01", "2022-12-31"),
        domains=["Legal", "Government", "Written"],
        dialect=[],
        task_subtypes=[],
        license="apache-2.0",
        annotations_creators="expert-annotated",
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{10.1145/3594536.3595177,
  address = {New York, NY, USA},
  author = {Licari, Daniele and Bushipaka, Praveen and Marino, Gabriele and Comand\'{e}, Giovanni and Cucinotta, Tommaso},
  booktitle = {Proceedings of the Nineteenth International Conference on Artificial Intelligence and Law},
  doi = {10.1145/3594536.3595177},
  isbn = {9798400701979},
  keywords = {Italian-LEGAL-BERT, Holding Extraction, Extractive Text Summarization, Benchmark Dataset},
  location = {<conf-loc>, <city>Braga</city>, <country>Portugal</country>, </conf-loc>},
  numpages = {9},
  pages = {148–156},
  publisher = {Association for Computing Machinery},
  series = {ICAIL '23},
  title = {Legal Holding Extraction from Italian Case Documents using Italian-LEGAL-BERT Text Summarization},
  url = {https://doi.org/10.1145/3594536.3595177},
  year = {2023},
}
""",
    )

    def dataset_transform(self):
        self.dataset = self.dataset.rename_columns(
            {"summary": "text", "materia": "label"}
        )



from mteb.abstasks.task_metadata import TaskMetadata
from mteb.abstasks.zeroshot_classification import (
    AbsTaskZeroShotClassification,
)


class MNISTZeroShotClassification(AbsTaskZeroShotClassification):
    metadata = TaskMetadata(
        name="MNISTZeroShot",
        description="Classifying handwritten digits.",
        reference="https://en.wikipedia.org/wiki/MNIST_database",
        dataset={
            "path": "ylecun/mnist",
            "revision": "77f3279092a1c1579b2250db8eafed0ad422088c",
        },
        type="ZeroShotClassification",
        category="i2t",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=(
            "2010-01-01",
            "2010-04-01",
        ),  # Estimated range for the collection of reviews
        domains=["Encyclopaedic"],
        task_subtypes=["Object recognition"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        modalities=["image", "text"],
        sample_creation="created",
        bibtex_citation=r"""
@article{lecun2010mnist,
  author = {LeCun, Yann and Cortes, Corinna and Burges, CJ},
  journal = {ATT Labs [Online]. Available: http://yann.lecun.com/exdb/mnist},
  title = {MNIST handwritten digit database},
  volume = {2},
  year = {2010},
}
""",
    )

    def get_candidate_labels(self) -> list[str]:
        return [
            f"a photo of the number: '{name}'."
            for name in self.dataset["test"].features[self.label_column_name].names
        ]



from mteb.models.model_meta import ModelMeta
from mteb.models.sentence_transformer_wrapper import sentence_transformers_loader

from .e5_instruct import E5_MISTRAL_TRAINING_DATA

LGAI_EMBEDDING_TRAINING_DATA = {
    # source: https://arxiv.org/abs/2506.07438
    "ArguAna",
    "ELI5",
    "FEVER",
    "FiQA2018",
    "HotpotQA",
    "MSMARCO",
    "NQ",
    "QuoraDuplicateQuestions",
    "SciDocsReranking",
    "SQuAD",
    "StackOverflowDupQuestions",
    "AmazonCounterfactualClassification",
    "AmazonReviewsClassification",
    "Banking77Classification",
    "EmotionClassification",
    "ImdbClassification",
    "MTOPIntentClassification",
    "ToxicConversationsClassification",
    "TweetSentimentExtractionClassification",
    "ArxivClusteringS2S",
    "ArxivClusteringP2P",
    "BiorxivClusteringS2S",
    "BiorxivClusteringP2P",
    "MedrxivClusteringS2S",
    "MedrxivClusteringP2P",
    "RedditClusteringP2P",
    "RedditClustering",
    "StackExchangeClustering",
    "StackExchangeClusteringP2P",
    "TwentyNewsgroupsClustering",
    "STS12",
    "STS22",
    "STSBenchmark",
    "SNLI",
}

lgai_embedding_en = ModelMeta(
    loader=sentence_transformers_loader,
    name="annamodels/LGAI-Embedding-Preview",
    languages=[
        "eng-Latn",
    ],
    open_weights=True,
    revision="5e0b2316acc8c2e2941ded6b9cb200b1cb313e65",
    release_date="2025-06-11",
    n_parameters=7_110_000_000,
    memory_usage_mb=27125,
    embed_dim=4096,
    license="apache-2.0",
    max_tokens=32768,
    reference="https://huggingface.co/annamodels/LGAI-Embedding-Preview",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch"],
    use_instructions=True,
    public_training_code=None,
    public_training_data=None,
    adapted_from="mistralai/Mistral-7B-v0.1",
    training_datasets=E5_MISTRAL_TRAINING_DATA | LGAI_EMBEDDING_TRAINING_DATA,
    citation="""@misc{choi2025lgaiembeddingpreviewtechnicalreport,
      title={LGAI-EMBEDDING-Preview Technical Report},
      author={Jooyoung Choi and Hyun Kim and Hansol Jang and Changwook Jun and Kyunghoon Bae and Hyewon Choi and Stanley Jungkyu Choi and Honglak Lee and Chulmin Yun},
      year={2025},
      eprint={2506.07438},
      archivePrefix={arXiv},
      primaryClass={cs.CL},
      url={https://arxiv.org/abs/2506.07438},
}
""",
)



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class SlovakMovieReviewSentimentClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="SlovakMovieReviewSentimentClassification",
        description="User reviews of movies on the CSFD movie database, with 2 sentiment classes (positive, negative)",
        reference="https://arxiv.org/pdf/2304.01922",
        dataset={
            "path": "janko/sk_csfd-movie-reviews",
            "revision": "0c47583c9d339b3b6f89e4db76088af5f1ec8d39",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["slk-Latn"],
        main_score="accuracy",
        date=("2002-05-21", "2020-03-05"),
        dialect=[],
        domains=["Reviews", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="cc-by-nc-sa-4.0",
        annotations_creators="derived",
        sample_creation="found",
        bibtex_citation=r"""
@article{vstefanik2023resources,
  author = {{\v{S}}tef{\'a}nik, Michal and Kadl{\v{c}}{\'\i}k, Marek and Gramacki, Piotr and Sojka, Petr},
  journal = {arXiv preprint arXiv:2304.01922},
  title = {Resources and Few-shot Learners for In-context Learning in Slavic Languages},
  year = {2023},
}
""",
        superseded_by="SlovakMovieReviewSentimentClassification.v2",
    )

    def dataset_transform(self) -> None:
        self.dataset = self.dataset.rename_columns({"comment": "text"})

        self.dataset = self.stratified_subsampling(
            self.dataset, seed=self.seed, splits=["test"]
        )


class SlovakMovieReviewSentimentClassificationV2(AbsTaskClassification):
    metadata = TaskMetadata(
        name="SlovakMovieReviewSentimentClassification.v2",
        description="User reviews of movies on the CSFD movie database, with 2 sentiment classes (positive, negative) This version corrects errors found in the original data. For details, see [pull request](https://github.com/embeddings-benchmark/mteb/pull/2900)",
        reference="https://arxiv.org/pdf/2304.01922",
        dataset={
            "path": "mteb/slovak_movie_review_sentiment",
            "revision": "29a7405aabcfd4860a51ae6f65a5650d63108f26",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["slk-Latn"],
        main_score="accuracy",
        date=("2002-05-21", "2020-03-05"),
        dialect=[],
        domains=["Reviews", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="cc-by-nc-sa-4.0",
        annotations_creators="derived",
        sample_creation="found",
        bibtex_citation=r"""
@article{vstefanik2023resources,
  author = {{\v{S}}tef{\'a}nik, Michal and Kadl{\v{c}}{\'\i}k, Marek and Gramacki, Piotr and Sojka, Petr},
  journal = {arXiv preprint arXiv:2304.01922},
  title = {Resources and Few-shot Learners for In-context Learning in Slavic Languages},
  year = {2023},
}
""",
        adapted_from=["SlovakMovieReviewSentimentClassification"],
    )

    def dataset_transform(self) -> None:
        self.dataset = self.stratified_subsampling(
            self.dataset, seed=self.seed, splits=["test"]
        )



from mteb.abstasks.multilabel_classification import (
    AbsTaskMultilabelClassification,
)
from mteb.abstasks.task_metadata import TaskMetadata


class CovidDisinformationNLMultiLabelClassification(AbsTaskMultilabelClassification):
    metadata = TaskMetadata(
        name="CovidDisinformationNLMultiLabelClassification",
        dataset={
            "path": "clips/mteb-nl-COVID-19-disinformation",
            "revision": "7ad922bdef875db1f530847c6ffff05fc154f2e8",
        },
        description="The dataset is curated to address questions of interest to journalists, fact-checkers, "
        "social media platforms, policymakers, and the general public.",
        reference="https://aclanthology.org/2021.findings-emnlp.56.pdf",
        type="MultilabelClassification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["nld-Latn"],
        main_score="f1",
        date=("2020-01-01", "2021-04-01"),
        domains=["Web", "Social", "Written"],
        task_subtypes=[],
        license="cc-by-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{alam-etal-2021-fighting-covid,
  address = {Punta Cana, Dominican Republic},
  author = {Alam, Firoj  and
Shaar, Shaden  and
Dalvi, Fahim  and
Sajjad, Hassan  and
Nikolov, Alex  and
Mubarak, Hamdy  and
Da San Martino, Giovanni  and
Abdelali, Ahmed  and
Durrani, Nadir  and
Darwish, Kareem  and
Al-Homaid, Abdulaziz  and
Zaghouani, Wajdi  and
Caselli, Tommaso  and
Danoe, Gijs  and
Stolk, Friso  and
Bruntink, Britt  and
Nakov, Preslav},
  booktitle = {Findings of the Association for Computational Linguistics: EMNLP 2021},
  doi = {10.18653/v1/2021.findings-emnlp.56},
  editor = {Moens, Marie-Francine  and
Huang, Xuanjing  and
Specia, Lucia  and
Yih, Scott Wen-tau},
  month = nov,
  pages = {611--649},
  publisher = {Association for Computational Linguistics},
  title = {Fighting the {COVID}-19 Infodemic: Modeling the Perspective of Journalists, Fact-Checkers, Social Media Platforms, Policy Makers, and the Society},
  url = {https://aclanthology.org/2021.findings-emnlp.56/},
  year = {2021},
}
""",
        prompt={
            "query": "Classificeer COVID-19-gerelateerde sociale media-berichten in alle toepasselijke desinformatiecategorieën"
        },
    )

    def dataset_transform(self) -> None:
        labels = [
            "q2_label",
            "q3_label",
            "q4_label",
            "q5_label",
            "q6_label",
            "q7_label",
        ]
        _dataset = {}

        def map_labels(example):
            ml_labels = []
            for i, label in enumerate(labels):
                if example[label] == "yes":
                    ml_labels.append(i)
            return {"label": ml_labels}

        for split in self.dataset:
            self.dataset[split] = self.dataset[split].filter(
                lambda ex: ex["q1_label"] == "yes"
            )
            self.dataset[split] = self.dataset[split].map(map_labels)



import json
import logging
import sys
from collections.abc import Sequence
from pathlib import Path

from mteb.abstasks.abstask import AbsTask
from mteb.models.model_meta import ModelMeta
from mteb.results import BenchmarkResults, ModelResult, TaskResult
from mteb.types import ModelName, Revision

if sys.version_info >= (3, 13):
    from warnings import deprecated
else:
    from typing_extensions import deprecated

logger = logging.getLogger(__name__)


def _model_name_and_revision(
    revision_path: Path, fallback_to_path: bool
) -> tuple[ModelName, Revision] | None:
    model_meta = revision_path / "model_meta.json"
    model_path = revision_path.parent
    if not model_meta.exists() and fallback_to_path:
        logger.info(
            f"model_meta.json not found in {revision_path}, extracting model_name and revision from the path"
        )
        model_name, revision = model_path.name, revision_path.name
    elif not model_meta.exists():
        return None
    else:
        with model_meta.open("r") as f:
            model_meta_json = json.load(f)
            model_name = model_meta_json["name"]
            revision = model_meta_json["revision"]

    return model_name, revision


@deprecated(
    "`load_results` is deprecated and will be removed in future versions. "
    + "Please use the `ResultCache`'s `.load_results` method instead."
)
def load_results(
    results_repo: str = "https://github.com/embeddings-benchmark/results",
    download_latest: bool = True,
    models: Sequence[ModelMeta] | Sequence[str] | None = None,
    tasks: Sequence[AbsTask] | Sequence[str] | None = None,
    validate_and_filter: bool = True,
    require_model_meta: bool = True,
    only_main_score: bool = False,
) -> BenchmarkResults:
    """Loads the results from the latest version of the [results](https://github.com/embeddings-benchmark/results) repository.

    The results are cached locally in the MTEB_CACHE directory.
    This directory can be set using the MTEB_CACHE environment variable or defaults to "~/.cache/mteb".

    Args:
        results_repo: The URL of the results repository on GitHub. Defaults to "https://github.com/embeddings-benchmark/results".
        download_latest: If True it will update the existing version of the results cache. Defaults to True.
        models: A list of model names to load the results for. If None it loads the results for all models. Defaults to None.
        tasks: A list of task names to load the results for. If None it loads the results for all tasks. Defaults to None.
        require_model_meta: If True it will ignore results that do not have a model_meta.json file. Defaults to True. If false it will
            extract the model name and revision from the path.
        validate_and_filter: If True it will validate that the results object for the task contains the correct splits and filter out
            splits from the results object that are not default in the task metadata. Defaults to True.
        only_main_score: If True, only the main score will be loaded.

    Returns:
        A BenchmarkResults object containing the results for the specified models and tasks.
    """
    from mteb.cache import ResultCache

    cache = ResultCache()
    if download_latest:
        cache.download_from_remote(remote=results_repo, download_latest=download_latest)
    repo_directory = cache.cache_path
    model_paths = [p for p in (repo_directory / "results").glob("*") if p.is_dir()]
    model_paths += [
        p for p in (repo_directory / "remote" / "results").glob("*") if p.is_dir()
    ]

    if models is not None:
        models_to_keep = {}
        for model_path in models:
            if isinstance(model_path, ModelMeta):
                models_to_keep[model_path.name] = model_path.revision
            else:
                models_to_keep[model_path] = None
    else:
        models_to_keep = None

    task_names = {}
    if tasks is not None:
        for task in tasks:
            if isinstance(task, AbsTask):
                task_names[task.metadata.name] = task
            else:
                task_names[task] = None

    model_results = []
    for model_path in model_paths:
        model_revisions = model_path.glob("*")

        for revision_path in model_revisions:
            model_name_and_revision = _model_name_and_revision(
                revision_path, fallback_to_path=(not require_model_meta)
            )
            if model_name_and_revision is None:
                continue
            model_name, revision = model_name_and_revision

            model_name = model_name.replace("__", "/")
            if models_to_keep is not None and model_name not in models_to_keep:
                continue
            elif models_to_keep is not None and models_to_keep[model_name] is not None:
                if models_to_keep[model_name] != revision:
                    continue

            task_json_files = [
                f for f in revision_path.glob("*.json") if "model_meta.json" != f.name
            ]
            _results = []
            for f in task_json_files:
                task_res = TaskResult.from_disk(f)
                if only_main_score:
                    task_res = task_res.only_main_score()
                _results.append(task_res)

            # filter out tasks that are not in the tasks list
            if tasks is not None:
                _results = [r for r in _results if r.task_name in task_names]

            if validate_and_filter:
                filtered_results = []
                for r in _results:
                    try:
                        if task_names:
                            task = task_names[r.task_name]
                        else:
                            task = None
                        r = r.validate_and_filter_scores(task=task)
                        filtered_results.append(r)
                    except Exception as e:
                        logger.info(
                            f"Validation failed for {r.task_name} in {model_name} {revision}: {e}"
                        )
                _results = filtered_results
            model_results.append(
                ModelResult(
                    model_name=model_name,
                    model_revision=revision,
                    task_results=_results,
                )
            )

    return BenchmarkResults(model_results=model_results)



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class Quail(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="Quail",
        description="Measuring the ability to retrieve the groundtruth answers to reasoning task queries on Quail.",
        reference="https://text-machine.cs.uml.edu/lab2/projects/quail/",
        dataset={
            "path": "mteb/Quail",
            "revision": "516ac28de67229da337281f702310a08c9fe0a03",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2020-01-01", "2020-12-31"),
        domains=["Encyclopaedic", "Written"],
        task_subtypes=["Reasoning as Retrieval"],
        license="cc-by-nc-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{rogers2020getting,
  author = {Rogers, Anna and Kovaleva, Olga and Downey, Matthew and Rumshisky, Anna},
  booktitle = {Proceedings of the AAAI conference on artificial intelligence},
  number = {05},
  pages = {8722--8731},
  title = {Getting closer to AI complete question answering: A set of prerequisite real tasks},
  volume = {34},
  year = {2020},
}

@article{xiao2024rar,
  author = {Xiao, Chenghao and Hudson, G Thomas and Moubayed, Noura Al},
  journal = {arXiv preprint arXiv:2404.06347},
  title = {RAR-b: Reasoning as Retrieval Benchmark},
  year = {2024},
}
""",
        prompt={
            "query": "Given the following context and question, retrieve the correct answer."
        },
    )



import logging
import time
from typing import Any

import numpy as np
import torch
from tqdm.auto import tqdm

from mteb._requires_package import requires_package
from mteb.models.abs_encoder import AbsEncoder
from mteb.models.model_meta import ModelMeta
from mteb.models.models_protocols import PromptType

from .bge_models import bge_chinese_training_data
from .nvidia_models import nvidia_training_datasets

logger = logging.getLogger(__name__)


seed_1_5_training_data = (
    {
        "PAWSX",
        "QBQTC",
        "STSB",
        "TNews",
        "Waimai",
        "IFlyTek",
    }
    | bge_chinese_training_data
    | nvidia_training_datasets
)


class SeedTextEmbeddingModel(AbsEncoder):
    def __init__(
        self,
        model_name: str,
        max_tokens: int,
        tokenizer_name: str = "cl100k_base",
        embed_dim: int | None = None,
        available_embed_dims: list[int | None] = [None],
        **kwargs,
    ) -> None:
        """Wrapper for Seed embedding API."""
        requires_package(
            self,
            "volcenginesdkarkruntime",
            "ByteDance Seed",
            "pip install mteb[ark]",
        )
        from volcenginesdkarkruntime import Ark

        requires_package(
            self,
            "tiktoken",
            "ByteDance Seed",
            "pip install mteb[ark]",
        )
        import tiktoken

        self._client = Ark()
        self._model_name = model_name
        self._max_tokens = max_tokens
        self._embed_dim = embed_dim
        self._available_embed_dims = available_embed_dims
        self._encoding = tiktoken.get_encoding(tokenizer_name)

    def truncate_text_tokens(self, text: str) -> str:
        """Truncate a string to have `max_tokens` according to the given encoding.

        Returns:
            Truncated string.
        """
        truncated_sentence = self._encoding.encode(text)[: self._max_tokens]
        return self._encoding.decode(truncated_sentence)

    def _encode(
        self, inputs: list[str], task_name: str, prompt_type: PromptType | None = None
    ):
        assert (
            self._embed_dim is None or self._embed_dim in self._available_embed_dims
        ), (
            f"Available embed_dims are {self._available_embed_dims}, found {self._embed_dim}"
        )

        if prompt_type == PromptType("query") or prompt_type is None:
            if task_name in TASK_NAME_TO_INSTRUCTION:
                instruction = TASK_NAME_TO_INSTRUCTION[task_name]
            else:
                instruction = DEFAULT_INSTRUCTION
            inputs = [instruction + i for i in inputs]

        response = self._client.embeddings.create(
            input=inputs,
            model=self._model_name,
            encoding_format="float",
        )
        outputs = torch.tensor(
            [d.embedding for d in response.data], dtype=torch.bfloat16
        )
        if self._embed_dim is not None:
            outputs = outputs[:, : self._embed_dim]
        outputs = torch.nn.functional.normalize(outputs, p=2, dim=1)
        return outputs.float().tolist()

    def encode(
        self,
        sentences: list[str],
        task_name: str,
        prompt_type: PromptType | None = None,
        retries: int = 5,
        **kwargs: Any,
    ) -> np.ndarray:
        trimmed_sentences = []
        for sentence in sentences:
            encoded_sentence = self._encoding.encode(sentence)
            if len(encoded_sentence) > self._max_tokens:
                truncated_sentence = self.truncate_text_tokens(sentence)
                trimmed_sentences.append(truncated_sentence)
            else:
                trimmed_sentences.append(sentence)

        max_batch_size = kwargs.get("batch_size", 32)
        sublists = [
            trimmed_sentences[i : i + max_batch_size]
            for i in range(0, len(trimmed_sentences), max_batch_size)
        ]

        show_progress_bar = (
            False
            if "show_progress_bar" not in kwargs
            else kwargs.pop("show_progress_bar")
        )

        all_embeddings = []

        for i, sublist in enumerate(
            tqdm(sublists, leave=False, disable=not show_progress_bar)
        ):
            while retries > 0:
                try:
                    embedding = self._encode(sublist, task_name, prompt_type)
                    all_embeddings.extend(embedding)
                    break
                except Exception as e:
                    # Sleep due to too many requests
                    time.sleep(1)
                    logger.warning(
                        f"Retrying... {retries} retries left. Error: {str(e)}"
                    )
                    retries -= 1
                    if retries == 0:
                        raise e

        return np.array(all_embeddings)


TASK_NAME_TO_INSTRUCTION = {
    "ArguAna": "Instruct: Given a claim, find documents that refute the claim\nQuery: ",
    "ClimateFEVERHardNegatives": "Instruct: Given a claim about climate change, retrieve documents that support or refute the claim\nQuery: ",
    "FEVERHardNegatives": "Instruct: Given a claim, retrieve documents that support or refute the claim\nQuery: ",
    "FiQA2018": "Instruct: Given a financial question, retrieve user replies that best answer the question\nQuery: ",
    "HotpotQAHardNegatives": "Instruct: Given a multi-hop question, retrieve documents that can help answer the question\nQuery: ",
    "SCIDOCS": "Instruct: Given a scientific paper title, retrieve paper abstracts that are cited by the given paper\nQuery: ",
    "Touche2020Retrieval.v3": "Instruct: Given a question, retrieve detailed and persuasive arguments that answer the question\nQuery: ",
    "TRECCOVID": "Instruct: Given a query on COVID-19, retrieve documents that answer the query\nQuery: ",
    "AskUbuntuDupQuestions": "Instruct: Retrieve duplicate questions from AskUbuntu forum\nQuery: ",
    "MindSmallReranking": "Instruct: Retrieve relevant news articles based on user browsing history\nQuery: ",
    "SprintDuplicateQuestions": "Instruct: Retrieve duplicate questions from Sprint forum\nQuery: ",
    "TwitterSemEval2015": "Instruct: Retrieve tweets that are semantically similar to the given tweet\nQuery: ",
    "TwitterURLCorpus": "Instruct: Retrieve tweets that are semantically similar to the given tweet\nQuery: ",
    "CQADupstackGamingRetrieval": "Instruct: Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question\nQuery: ",
    "CQADupstackUnixRetrieval": "Instruct: Given a question, retrieve detailed question descriptions from Stackexchange that are duplicates to the given question\nQuery: ",
    "DuRetrieval": "Instruct: Given a Chinese search query, retrieve web passages that answer the question\nQuery: ",
    "T2Retrieval": "Instruct: Given a Chinese search query, retrieve web passages that answer the question\nQuery: ",
    "MMarcoRetrieval": "Instruct: Given a Chinese search query, retrieve web passages that answer the question\nQuery: ",
    "MMarcoReranking": "Instruct: Based on a Chinese search question, evaluate and rank the web passages that provide answers to the question\nQuery: ",
    "T2Reranking": "Instruct: Based on a Chinese search question, evaluate and rank the web passages that provide answers to the question\nQuery: ",
    "CMedQAv1-reranking": "Instruct: Based a Chinese medical question, evaluate and rank the medical information that provide answers to the question\nQuery: ",
    "CMedQAv2-reranking": "Instruct: Based a Chinese medical question, evaluate and rank the medical information that provide answers to the question\nQuery: ",
    "CovidRetrieval": "Instruct: Given a Chinese question on COVID-19, retrieve news articles that answer the question\nQuery: ",
    "CmedqaRetrieval": "Instruct: Given a Chinese query on medical information, retrieve relevant documents that answer the query\nQuery: ",
    "VideoRetrieval": "Instruct: Given a video search query, retrieve the titles of relevant videos\nQuery: ",
    "EcomRetrieval": "Instruct: Given a user query from an e-commerce website, retrieve description sentences of relevant products\nQuery: ",
    "MedicalRetrieval": "Instruct: Given a medical question, retrieve user replies that best answer the question\nQuery: ",
    "ATEC": "Instruct: Retrieve semantically similar text.\nQuery: ",
    "BQ": "Instruct: Retrieve semantically similar text.\nQuery: ",
    "LCQMC": "Instruct: Retrieve semantically similar text.\nQuery: ",
    "PAWSX": "Instruct: Retrieve semantically similar text.\nQuery: ",
    "STSB": "Instruct: Retrieve semantically similar text.\nQuery: ",
    "AFQMC": "Instruct: Retrieve semantically similar text.\nQuery: ",
    "QBQTC": "Instruct: Retrieve semantically similar text.\nQuery: ",
    "STS22.v2": "Instruct: Retrieve semantically similar text.\nQuery: ",
    "BIOSSES": "Instruct: Retrieve semantically similar text.\nQuery: ",
    "SICK-R": "Instruct: Retrieve semantically similar text.\nQuery: ",
    "STS12": "Instruct: Retrieve semantically similar text.\nQuery: ",
    "STS13": "Instruct: Retrieve semantically similar text.\nQuery: ",
    "STS14": "Instruct: Retrieve semantically similar text.\nQuery: ",
    "STS15": "Instruct: Retrieve semantically similar text.\nQuery: ",
    "STS17": "Instruct: Retrieve semantically similar text.\nQuery: ",
    "STSBenchmark": "Instruct: Retrieve semantically similar text.\nQuery: ",
    "SummEvalSummarization.v2": "Instruct: Retrieve semantically similar text.\nQuery: ",
    "Ocnli": "Instruct: Given a premise, retrieve a hypothesis that is entailed by the premise\nQuery: ",
    "Cmnli": "Instruct: Given a premise, retrieve a hypothesis that is entailed by the premise\nQuery: ",
    "TNews": "Instruct: Classify the fine-grained category of the given news title\nQuery: ",
    "IFlyTek": "Instruct: Given an App description text, find the appropriate fine-grained category\nQuery: ",
    "MultilingualSentiment": "Instruct: Classify sentiment of the customer review into positive, neutral, or negative\nQuery: ",
    "JDReview": "Instruct: Classify the customer review for iPhone on e-commerce platform into positive or negative\nQuery: ",
    "OnlineShopping": "Instruct: Classify the customer review for online shopping into positive or negative\nQuery: ",
    "Waimai": "Instruct: Classify the customer review from a food takeaway platform into positive or negative\nQuery: ",
    "AmazonCounterfactualClassification": "Instruct: Classify a given Amazon customer review text as either counterfactual or not-counterfactual\nQuery: ",
    "AmazonReviewsClassification": "Instruct: Classify a given Amazon review into its appropriate rating category\nQuery: ",
    "Banking77Classification": "Instruct: Given an online banking query, find the corresponding intents\nQuery: ",
    "ImdbClassification": "Instruct: Classify the sentiment expressed in a given movie review text from the IMDB dataset\nQuery: ",
    "MassiveIntentClassification": "Instruct: Given a user utterance as query, find the user intents\nQuery: ",
    "MassiveScenarioClassification": "Instruct: Given a user utterance as query, find the user scenarios\nQuery: ",
    "MTOPDomainClassification": "Instruct: Classify the intent domain of a given utterance in task-oriented conversation\nQuery: ",
    "ToxicConversationsClassification": "Instruct: Classify the given comments as either toxic or not toxic\nQuery: ",
    "TweetSentimentExtractionClassification": "Instruct: Classify the sentiment of a given tweet as either positive, negative, or neutral\nQuery: ",
    "ArXivHierarchicalClusteringP2P": "Instruct: Identify the main and secondary category of Arxiv papers based on the titles and abstracts\nQuery: ",
    "ArXivHierarchicalClusteringS2S": "Instruct: Identify the main and secondary category of Arxiv papers based on the titles\nQuery: ",
    "BiorxivClusteringP2P.v2": "Instruct: Identify the main category of Biorxiv papers based on the titles and abstracts\nQuery: ",
    "MedrxivClusteringP2P.v2": "Instruct: Identify the main category of Medrxiv papers based on the titles and abstracts\nQuery: ",
    "MedrxivClusteringS2S.v2": "Instruct: Identify the main category of Medrxiv papers based on the titles\nQuery: ",
    "StackExchangeClustering.v2": "Instruct: Identify the topic or theme of StackExchange posts based on the titles\nQuery: ",
    "StackExchangeClusteringP2P.v2": "Instruct: Identify the topic or theme of StackExchange posts based on the given paragraphs\nQuery: ",
    "TwentyNewsgroupsClustering.v2": "Instruct: Identify the topic or theme of the given news articles\nQuery: ",
    "CLSClusteringS2S": "Instruct: Identify the main category of scholar papers based on the titles\nQuery: ",
    "CLSClusteringP2P": "Instruct: Identify the main category of scholar papers based on the titles and abstracts\nQuery: ",
    "ThuNewsClusteringS2S": "Instruct: Identify the topic or theme of the given news articles based on the titles\nQuery: ",
    "ThuNewsClusteringP2P": "Instruct: Identify the topic or theme of the given news articles based on the titles and abstracts\nQuery: ",
}

DEFAULT_INSTRUCTION = "Instruct: Given a web search query, retrieve relevant passages that answer the query\nQuery: "


seed_embedding = ModelMeta(
    name="ByteDance-Seed/Seed1.5-Embedding",
    revision="4",
    release_date="2025-04-25",
    languages=[
        "eng-Latn",
        "zho-Hans",
    ],
    loader=SeedTextEmbeddingModel,  # type: ignore[call-arg]
    loader_kwargs=dict(
        model_name="doubao-embedding-large-text-250515",
        max_tokens=32000,  # tiktoken
        available_embed_dims=[2048, 1024, 512, 256],
    ),
    max_tokens=32768,
    embed_dim=2048,
    open_weights=False,
    n_parameters=None,
    memory_usage_mb=None,
    license=None,
    reference="https://seed1-5-embedding.github.io/",
    similarity_fn_name="cosine",  # type: ignore[arg-type]
    framework=["API"],
    use_instructions=True,
    training_datasets=seed_1_5_training_data,
    public_training_code=None,
    public_training_data=None,
)



from .ajgt import AJGT, AJGTV2
from .hotel_review_sentiment_classification import (
    HotelReviewSentimentClassification,
    HotelReviewSentimentClassificationV2,
)
from .online_store_review_sentiment_classification import (
    OnlineStoreReviewSentimentClassification,
    OnlineStoreReviewSentimentClassificationV2,
)
from .restaurant_review_sentiment_classification import (
    RestaurantReviewSentimentClassification,
    RestaurantReviewSentimentClassificationV2,
)
from .tweet_emotion_classification import (
    TweetEmotionClassification,
    TweetEmotionClassificationV2,
)
from .tweet_sarcasm_classification import (
    TweetSarcasmClassification,
    TweetSarcasmClassificationV2,
)

__all__ = [
    "AJGT",
    "AJGTV2",
    "HotelReviewSentimentClassification",
    "HotelReviewSentimentClassificationV2",
    "OnlineStoreReviewSentimentClassification",
    "OnlineStoreReviewSentimentClassificationV2",
    "RestaurantReviewSentimentClassification",
    "RestaurantReviewSentimentClassificationV2",
    "TweetEmotionClassification",
    "TweetEmotionClassificationV2",
    "TweetSarcasmClassification",
    "TweetSarcasmClassificationV2",
]



import torch

from mteb.models.instruct_wrapper import (
    InstructSentenceTransformerModel,
    instruct_wrapper,
)
from mteb.models.model_meta import ModelMeta, ScoringFunction

from .e5_models import (
    E5_PAPER_RELEASE_DATE,
    ME5_TRAINING_DATA,
    XLMR_LANGUAGES,
)

MISTRAL_LANGUAGES = ["eng-Latn", "fra-Latn", "deu-Latn", "ita-Latn", "spa-Latn"]

E5_INSTRUCTION = "Instruct: {instruction}\nQuery: "

E5_MISTRAL_TRAINING_DATA = {
    "FEVER",
    "FEVERHardNegatives",
    "FEVER-NL",  # translation not trained on
    "HotpotQA",
    "HotpotQAHardNegatives",
    "HotpotQA-PL",  # translation not trained on
    "HotpotQA-NL",  # translation not trained on
    "MIRACLRetrieval",
    "MIRACLRetrievalHardNegatives",
    "MIRACLReranking",  # https://arxiv.org/pdf/2402.09906, section M
} | ME5_TRAINING_DATA

e5_instruct = ModelMeta(
    loader=instruct_wrapper,
    loader_kwargs=dict(
        instruction_template=E5_INSTRUCTION,
        attn="cccc",
        pooling_method="mean",
        mode="embedding",
        torch_dtype=torch.float16,
        normalized=True,
    ),
    name="intfloat/multilingual-e5-large-instruct",
    languages=XLMR_LANGUAGES,
    open_weights=True,
    revision="baa7be480a7de1539afce709c8f13f833a510e0a",
    release_date=E5_PAPER_RELEASE_DATE,
    framework=["GritLM", "PyTorch", "Sentence Transformers"],
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=True,
    reference="https://huggingface.co/intfloat/multilingual-e5-large-instruct",
    n_parameters=560_000_000,
    memory_usage_mb=1068,
    embed_dim=1024,
    license="mit",
    max_tokens=514,
    adapted_from="FacebookAI/xlm-roberta-large",
    citation="""@article{wang2024multilingual,
      title={Multilingual E5 Text Embeddings: A Technical Report},
      author={Wang, Liang and Yang, Nan and Huang, Xiaolong and Yang, Linjun and Majumder, Rangan and Wei, Furu},
      journal={arXiv preprint arXiv:2402.05672},
      year={2024}
    }""",
    public_training_code=None,
    public_training_data=None,
    training_datasets=ME5_TRAINING_DATA,
)

e5_mistral = ModelMeta(
    loader=instruct_wrapper,
    loader_kwargs=dict(
        instruction_template=E5_INSTRUCTION,
        attn="cccc",
        pooling_method="lasttoken",
        mode="embedding",
        torch_dtype=torch.float16,
        # The ST script does not normalize while the HF one does so unclear what to do
        # https://huggingface.co/intfloat/e5-mistral-7b-instruct#transformers
        normalized=True,
    ),
    name="intfloat/e5-mistral-7b-instruct",
    languages=MISTRAL_LANGUAGES,
    open_weights=True,
    revision="07163b72af1488142a360786df853f237b1a3ca1",
    release_date=E5_PAPER_RELEASE_DATE,
    framework=["GritLM", "PyTorch", "Sentence Transformers"],
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=True,
    reference="https://huggingface.co/intfloat/e5-mistral-7b-instruct",
    n_parameters=7_111_000_000,
    memory_usage_mb=13563,
    embed_dim=4096,
    license="mit",
    max_tokens=32768,
    citation="""
    @article{wang2023improving,
      title={Improving Text Embeddings with Large Language Models},
      author={Wang, Liang and Yang, Nan and Huang, Xiaolong and Yang, Linjun and Majumder, Rangan and Wei, Furu},
      journal={arXiv preprint arXiv:2401.00368},
      year={2023}
    }

    @article{wang2022text,
      title={Text Embeddings by Weakly-Supervised Contrastive Pre-training},
      author={Wang, Liang and Yang, Nan and Huang, Xiaolong and Jiao, Binxing and Yang, Linjun and Jiang, Daxin and Majumder, Rangan and Wei, Furu},
      journal={arXiv preprint arXiv:2212.03533},
      year={2022}
    }
    """,
    public_training_code=None,
    public_training_data=None,
    training_datasets=E5_MISTRAL_TRAINING_DATA,
    adapted_from="mistralai/Mistral-7B-v0.1",
)

zeta_alpha_ai__zeta_alpha_e5_mistral = ModelMeta(
    loader=instruct_wrapper,
    loader_kwargs=dict(
        instruction_template=E5_INSTRUCTION,
        attn="cccc",
        pooling_method="lasttoken",
        mode="embedding",
        torch_dtype=torch.bfloat16,
        # The ST script does not normalize while the HF one does so unclear what to do
        # https://huggingface.co/intfloat/e5-mistral-7b-instruct#transformers
        normalized=True,
    ),
    name="zeta-alpha-ai/Zeta-Alpha-E5-Mistral",
    revision="c791d37474fa6a5c72eb3a2522be346bc21fbfc3",
    release_date="2024-08-30",
    languages=["eng-Latn"],
    n_parameters=7110660096,
    memory_usage_mb=13563,
    max_tokens=32768.0,
    embed_dim=4096,
    license="mit",
    open_weights=True,
    public_training_data=None,
    public_training_code=None,
    framework=["PyTorch", "Sentence Transformers", "GritLM"],
    reference="https://huggingface.co/zeta-alpha-ai/Zeta-Alpha-E5-Mistral",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=True,
    training_datasets={
        # copied from e5
        # source: https://arxiv.org/pdf/2212.03533
        "NQ",
        "NQ-NL",  # translation not trained on
        "NQHardNegatives",
        "MSMARCO",  # dev?
        "mMARCO-NL",  # translation not trained on
        # source: https://www.zeta-alpha.com/post/fine-tuning-an-llm-for-state-of-the-art-retrieval-zeta-alpha-s-top-10-submission-to-the-the-mteb-be
        # "Arguana",
        # "FEVER",
        # "FIQA",
        # "HotPotQA",
        # "MsMarco (passage)",
        # "NFCorpus",
        # "SciFact",
        # "NLI",
        # "SQuad",
        # "StackExchange",
        # "TriviaQA",
        # "SciRep",
        # "SciRepEval"
        # mteb
        # https://huggingface.co/datasets/mteb/raw_arxiv
        # "ArxivClusteringS2S",
        # "ArxivClusteringP2P",
        # https://huggingface.co/datasets/mteb/raw_biorxiv
        # "BiorxivClusteringS2S",
        # "BiorxivClusteringP2P",
        # https://huggingface.co/datasets/mteb/raw_medrxiv
        # "MedrxivClusteringS2S",
        # "MedrxivClusteringP2P",
        # as their train datasets
        "AmazonCounterfactualClassification",
        "AmazonReviewsClassification",
        "Banking77Classification",
        "EmotionClassification",
        "MTOPIntentClassification",
        "ToxicConversationsClassification",
        "TweetSentimentExtractionClassification",
        "ImdbClassification",
        "STS12",
        "STS22",
        "STSBenchmark",
        "MIRACLRetrieval",
        "MIRACLRetrievalHardNegatives",
        "MIRACLReranking",  # https://arxiv.org/pdf/2402.05672, table 2
    }
    | E5_MISTRAL_TRAINING_DATA,
    adapted_from="intfloat/e5-mistral-7b-instruct",
    superseded_by=None,
)

E5_R_MISTRAL_7B_INSTRUCTION = "{instruction}\n"
BeastyZ__e5_R_mistral_7b = ModelMeta(
    loader=InstructSentenceTransformerModel,
    loader_kwargs=dict(
        instruction_template=E5_R_MISTRAL_7B_INSTRUCTION,
        tokenizer_kwargs={"pad_token": "</s>"},
    ),
    name="BeastyZ/e5-R-mistral-7b",
    revision="3f810a6a7fd220369ad248e3705cf13d71803602",
    release_date="2024-06-28",
    languages=["eng-Latn"],
    n_parameters=7241732096,
    memory_usage_mb=27625,
    max_tokens=32768.0,
    embed_dim=4096,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/LeeSureman/E5-Retrieval-Reproduction",
    public_training_data="https://huggingface.co/datasets/BeastyZ/E5-R",
    framework=["PyTorch"],
    reference="https://huggingface.co/BeastyZ/e5-R-mistral-7b",
    similarity_fn_name="cosine",
    use_instructions=True,
    training_datasets=E5_MISTRAL_TRAINING_DATA,
    # not MTEB: {"BeastyZ/E5-R"},
    adapted_from="intfloat/e5-mistral-7b-instruct",
    superseded_by=None,
)



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class YelpReviewFullClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="YelpReviewFullClassification",
        description="Yelp Review Full is a dataset for sentiment analysis, containing 5 classes corresponding to ratings 1-5.",
        reference="https://arxiv.org/abs/1509.01626",
        dataset={
            "path": "Yelp/yelp_review_full",
            "revision": "c1f9ee939b7d05667af864ee1cb066393154bf85",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=("2015-01-01", "2015-12-31"),  # reviews from 2015
        domains=["Reviews", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="https://huggingface.co/datasets/Yelp/yelp_review_full#licensing-information",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{NIPS2015_250cf8b5,
  author = {Zhang, Xiang and Zhao, Junbo and LeCun, Yann},
  booktitle = {Advances in Neural Information Processing Systems},
  editor = {C. Cortes and N. Lawrence and D. Lee and M. Sugiyama and R. Garnett},
  pages = {},
  publisher = {Curran Associates, Inc.},
  title = {Character-level Convolutional Networks for Text Classification},
  url = {https://proceedings.neurips.cc/paper_files/paper/2015/file/250cf8b51c773f3f8dc8b4be867a9a02-Paper.pdf},
  volume = {28},
  year = {2015},
}
""",
        superseded_by="YelpReviewFullClassification.v2",
    )

    samples_per_label = 128

    def dataset_transform(self):
        self.dataset = self.stratified_subsampling(
            self.dataset, seed=self.seed, splits=["test"]
        )


class YelpReviewFullClassificationV2(AbsTaskClassification):
    metadata = TaskMetadata(
        name="YelpReviewFullClassification.v2",
        description="Yelp Review Full is a dataset for sentiment analysis, containing 5 classes corresponding to ratings 1-5. This version corrects errors found in the original data. For details, see [pull request](https://github.com/embeddings-benchmark/mteb/pull/2900)",
        reference="https://arxiv.org/abs/1509.01626",
        dataset={
            "path": "mteb/yelp_review_full",
            "revision": "49d71141934ae2e58733acd90908140e8ecaaee0",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=("2015-01-01", "2015-12-31"),  # reviews from 2015
        domains=["Reviews", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="https://huggingface.co/datasets/Yelp/yelp_review_full#licensing-information",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{NIPS2015_250cf8b5,
  author = {Zhang, Xiang and Zhao, Junbo and LeCun, Yann},
  booktitle = {Advances in Neural Information Processing Systems},
  editor = {C. Cortes and N. Lawrence and D. Lee and M. Sugiyama and R. Garnett},
  pages = {},
  publisher = {Curran Associates, Inc.},
  title = {Character-level Convolutional Networks for Text Classification},
  url = {https://proceedings.neurips.cc/paper_files/paper/2015/file/250cf8b51c773f3f8dc8b4be867a9a02-Paper.pdf},
  volume = {28},
  year = {2015},
}
""",
        adapted_from=["YelpReviewFullClassification"],
    )

    samples_per_label = 128

    def dataset_transform(self):
        self.dataset = self.stratified_subsampling(
            self.dataset, seed=self.seed, splits=["test"]
        )



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class IconclassClassification(AbsTaskClassification):
    samples_per_label = 32
    metadata = TaskMetadata(
        name="IconclassClassification",
        description="Iconclass is an iconographic thesaurus, which is widely used in the digital heritage domain to "
        "describe subjects depicted in artworks. The task is to classify the first layer of Iconclass",
        reference="https://dl.acm.org/doi/pdf/10.1145/3575865",
        dataset={
            "path": "clips/mteb-nl-iconclass-cls",
            "revision": "1cd02f1579dab39fedc95de8cc15fd620557a9f2",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        date=("2020-01-01", "2020-05-01"),
        eval_splits=["test"],
        eval_langs=["nld-Latn"],
        main_score="f1",
        domains=["Written", "Fiction"],
        task_subtypes=[],
        license="cc-by-nc-sa-4.0",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@article{banar2023transfer,
  author = {Banar, Nikolay and Daelemans, Walter and Kestemont, Mike},
  journal = {ACM Journal on Computing and Cultural Heritage},
  number = {2},
  pages = {1--16},
  publisher = {ACM New York, NY},
  title = {Transfer learning for the visual arts: The multi-modal retrieval of iconclass codes},
  volume = {16},
  year = {2023},
}
""",
        prompt={
            "query": "Classificeer de gegeven titel van het kunstwerk in het juiste onderwerp of thema"
        },
    )



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class PunjabiNewsClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="PunjabiNewsClassification",
        description="A Punjabi dataset for 2-class classification of Punjabi news articles",
        reference="https://github.com/goru001/nlp-for-punjabi/",
        dataset={
            "path": "mlexplorer008/punjabi_news_classification",
            "revision": "cec3923e16519efe51d535497e711932b8f1dc44",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        date=("2014-01-01", "2018-01-01"),
        eval_splits=["test"],
        eval_langs=["pan-Guru"],
        main_score="accuracy",
        domains=["News", "Written"],
        task_subtypes=["Topic classification"],
        license="mit",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@article{kunchukuttan2020indicnlpcorpus,
  author = {Anoop Kunchukuttan and Divyanshu Kakwani and Satish Golla and Gokul N.C. and Avik Bhattacharyya and Mitesh M. Khapra and Pratyush Kumar},
  journal = {arXiv preprint arXiv:2005.00085},
  title = {AI4Bharat-IndicNLP Corpus: Monolingual Corpora and Word Embeddings for Indic Languages},
  year = {2020},
}
""",
    )

    def dataset_transform(self):
        self.dataset = self.dataset.rename_columns(
            {"article": "text", "is_about_politics": "label"}
        )



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class BLINKIT2IRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="BLINKIT2IRetrieval",
        description="Retrieve images based on images and specific retrieval instructions.",
        reference="https://arxiv.org/abs/2404.12390",
        dataset={
            "path": "JamieSJS/blink-it2i",
            "revision": "7a1a1330565faca9c1aeec6f5acfc64f21296753",
        },
        type="Any2AnyRetrieval",
        category="it2i",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="cv_recall_at_1",
        date=("2018-01-01", "2018-12-31"),
        domains=["Encyclopaedic"],
        task_subtypes=["Image Text Retrieval"],
        license="cc-by-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        modalities=["text", "image"],
        sample_creation="found",
        bibtex_citation=r"""
@article{fu2024blink,
  author = {Fu, Xingyu and Hu, Yushi and Li, Bangzheng and Feng, Yu and Wang, Haoyu and Lin, Xudong and Roth, Dan and Smith, Noah A and Ma, Wei-Chiu and Krishna, Ranjay},
  journal = {arXiv preprint arXiv:2404.12390},
  title = {Blink: Multimodal large language models can see but not perceive},
  year = {2024},
}
""",
    )



import datasets

from mteb.abstasks.image.image_text_pair_classification import (
    AbsTaskImageTextPairClassification,
)
from mteb.abstasks.task_metadata import TaskMetadata


class SugarCrepe(AbsTaskImageTextPairClassification):
    images_column_names = ("images",)
    texts_column_names = ("caption", "negative_caption")

    metadata = TaskMetadata(
        name="SugarCrepe",
        description="Compositionality Evaluation of images to their captions.",
        reference="https://proceedings.neurips.cc/paper_files/paper/2023/hash/63461de0b4cb760fc498e85b18a7fe81-Abstract-Datasets_and_Benchmarks.html",
        dataset={
            "path": "yjkimstats/SUGARCREPE_fmt",
            "revision": "134abf9ade6a32f9fdae0e89022ff227a70b87e5",
        },
        type="Compositionality",
        category="i2t",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="text_acc",
        date=(
            "2022-01-01",
            "2022-12-31",
        ),  # Estimated range for the collection of data
        domains=["Encyclopaedic"],
        task_subtypes=["Caption Pairing"],
        license="mit",
        annotations_creators="expert-annotated",
        dialect=[],
        modalities=["text", "image"],
        sample_creation="created",
        bibtex_citation=r"""
@article{hsieh2024sugarcrepe,
  author = {Hsieh, Cheng-Yu and Zhang, Jieyu and Ma, Zixian and Kembhavi, Aniruddha and Krishna, Ranjay},
  journal = {Advances in neural information processing systems},
  title = {Sugarcrepe: Fixing hackable benchmarks for vision-language compositionality},
  volume = {36},
  year = {2024},
}
""",
    )

    def load_data(self) -> None:
        """Load dataset from HuggingFace hub"""
        if self.data_loaded:
            return
        self.dataset = datasets.load_dataset(**self.metadata.dataset)  # type: ignore
        self.dataset = datasets.DatasetDict({"test": self.dataset["train"]})
        self.dataset_transform()
        self.data_loaded = True



from .ria_news_retrieval import (
    RiaNewsRetrieval,
    RiaNewsRetrievalHardNegatives,
    RiaNewsRetrievalHardNegativesV2,
)
from .ru_bq_retrieval import RuBQRetrieval

__all__ = [
    "RiaNewsRetrieval",
    "RiaNewsRetrievalHardNegatives",
    "RiaNewsRetrievalHardNegativesV2",
    "RuBQRetrieval",
]



import pytest

from mteb import get_tasks
from mteb.abstasks.abstask import AbsTask
from mteb.abstasks.task_metadata import TaskDomain, TaskType
from mteb.filter_tasks import filter_tasks
from mteb.tasks.aggregated_tasks import CQADupstackRetrieval
from mteb.types import Modalities


@pytest.fixture
def all_tasks():
    return get_tasks()


def test_get_tasks_size_differences(all_tasks: list[AbsTask]):
    assert len(all_tasks) > 0
    assert len(all_tasks) >= len(filter_tasks(all_tasks, script=["Latn"]))
    assert len(all_tasks) >= len(filter_tasks(all_tasks, domains=["Legal"]))
    assert len(all_tasks) >= len(filter_tasks(all_tasks, languages=["eng", "deu"]))
    text_task = filter_tasks(all_tasks, modalities=["text"])
    assert len(all_tasks) >= len(text_task)
    assert len(filter_tasks(all_tasks, modalities=["text", "image"])) >= len(text_task)


@pytest.mark.parametrize("languages", [["eng", "deu"], ["eng"], None])
@pytest.mark.parametrize("script", [["Latn"], ["Cyrl"], None])
@pytest.mark.parametrize("domains", [["Legal"], ["Medical", "Non-fiction"], None])
@pytest.mark.parametrize("task_types", [["Classification"], None])
def test_filter_tasks(
    all_tasks: list[AbsTask],
    languages: list[str],
    script: list[str],
    domains: list[TaskDomain],
    task_types: list[TaskType] | None,  # type: ignore
):
    """Tests that get_tasks filters tasks correctly. This could in principle be combined with the following tests, but they have been kept
    separate to reduce the grid size.
    """
    tasks = filter_tasks(
        all_tasks,
        languages=languages,
        script=script,
        domains=domains,
        task_types=task_types,
    )

    for task in tasks:
        if languages:
            assert set(languages).intersection(task.metadata.languages)
        if script:
            assert set(script).intersection(task.metadata.scripts)
        if domains:
            task_domains = (
                set(task.metadata.domains) if task.metadata.domains else set()
            )
            assert set(domains).intersection(set(task_domains))
        if task_types:
            assert task.metadata.type in task_types


@pytest.mark.parametrize("languages", [["eng", "deu"], ["eng"]])
@pytest.mark.parametrize("domains", [["Medical", "Non-fiction"], None])
@pytest.mark.parametrize("task_types", [["Classification"], None])
@pytest.mark.parametrize("exclude_superseded_datasets", [True, False])
def test_filter_tasks_superseded(
    all_tasks: list[AbsTask],
    languages: list[str],
    domains: list[TaskDomain],
    task_types: list[TaskType] | None,  # type: ignore
    exclude_superseded_datasets: bool,
):
    tasks = filter_tasks(
        all_tasks,
        languages=languages,
        domains=domains,
        task_types=task_types,
        exclude_superseded=exclude_superseded_datasets,
    )

    for task in tasks:
        if languages:
            assert set(languages).intersection(task.metadata.languages)
        if domains:
            task_domains = (
                set(task.metadata.domains) if task.metadata.domains else set()
            )
            assert set(domains).intersection(set(task_domains))
        if task_types:
            assert task.metadata.type in task_types
        if exclude_superseded_datasets:
            assert task.superseded_by is None


@pytest.mark.parametrize("languages", [["eng", "deu"], ["eng"]])
@pytest.mark.parametrize("modalities", [["text"], ["image"], ["text", "image"], None])
@pytest.mark.parametrize("exclusive_modality_filter", [True, False])
def test_filter_tasks_modalities(
    all_tasks: list[AbsTask],
    languages: list[str],
    modalities: list[Modalities] | None,
    exclusive_modality_filter: bool,
):
    tasks = filter_tasks(
        all_tasks,
        languages=languages,
        modalities=modalities,
        exclusive_modality_filter=exclusive_modality_filter,
    )

    for task in tasks:
        if languages:
            assert set(languages).intersection(task.metadata.languages)
        if modalities:
            if exclusive_modality_filter:
                assert set(task.modalities) == set(modalities)
            else:
                assert any(mod in task.modalities for mod in modalities)


@pytest.mark.parametrize("languages", [["eng", "deu"], ["eng"], None])
@pytest.mark.parametrize("script", [["Latn"], ["Cyrl"], None])
@pytest.mark.parametrize("exclude_aggregate", [True, False])
def test_filter_tasks_exclude_aggregate(
    all_tasks: list[AbsTask],
    languages: list[str],
    script: list[str],
    exclude_aggregate: bool,
):
    tasks = filter_tasks(
        all_tasks,
        languages=languages,
        script=script,
        exclude_aggregate=exclude_aggregate,
    )

    for task in tasks:
        if languages:
            assert set(languages).intersection(task.metadata.languages)
        if script:
            assert set(script).intersection(task.metadata.scripts)
        if exclude_aggregate:
            # Aggregate tasks should be excluded
            assert not task.is_aggregate


def test_filter_tasks_exclude_aggregate_with_task_classes():
    """Test that filter_tasks correctly handles aggregate tasks when passed task classes.

    Regression test for PR #3372
    """
    tasks = filter_tasks([CQADupstackRetrieval], exclude_aggregate=True)
    assert len(tasks) == 0

    tasks = filter_tasks([CQADupstackRetrieval], exclude_aggregate=False)
    assert len(tasks) == 1



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class BengaliDocumentClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="BengaliDocumentClassification",
        description="Dataset for News Classification, categorized with 13 domains.",
        reference="https://aclanthology.org/2023.eacl-main.4",
        dataset={
            "path": "dialect-ai/shironaam",
            "revision": "1c6e67433da618073295b7c90f1c55fa8e78f35c",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["ben-Beng"],
        main_score="accuracy",
        date=("2022-05-01", "2023-05-01"),
        dialect=[],
        domains=["News", "Written"],
        task_subtypes=[],
        license="cc-by-nc-sa-4.0",
        annotations_creators="derived",
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{akash-etal-2023-shironaam,
  address = {Dubrovnik, Croatia},
  author = {Akash, Abu Ubaida  and
Nayeem, Mir Tafseer  and
Shohan, Faisal Tareque  and
Islam, Tanvir},
  booktitle = {Proceedings of the 17th Conference of the European Chapter of the Association for Computational Linguistics},
  month = may,
  pages = {52--67},
  publisher = {Association for Computational Linguistics},
  title = {Shironaam: {B}engali News Headline Generation using Auxiliary Information},
  url = {https://aclanthology.org/2023.eacl-main.4},
  year = {2023},
}
""",
        superseded_by="BengaliDocumentClassification.v2",
    )

    def dataset_transform(self) -> None:
        self.dataset = self.dataset.rename_columns(
            {"article": "text", "category": "label"}
        )
        self.dataset = self.stratified_subsampling(
            self.dataset, seed=self.seed, splits=["test"]
        )


class BengaliDocumentClassificationV2(AbsTaskClassification):
    metadata = TaskMetadata(
        name="BengaliDocumentClassification.v2",
        description="Dataset for News Classification, categorized with 13 domains. This version corrects errors found in the original data. For details, see [pull request](https://github.com/embeddings-benchmark/mteb/pull/2632)",
        reference="https://aclanthology.org/2023.eacl-main.4",
        dataset={
            "path": "mteb/bengali_document",
            "revision": "c5427220bbc2bc0ab6e19dabdbbd6d4e82b4846a",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["ben-Beng"],
        main_score="accuracy",
        date=("2022-05-01", "2023-05-01"),
        dialect=[],
        domains=["News", "Written"],
        task_subtypes=[],
        license="cc-by-nc-sa-4.0",
        annotations_creators="derived",
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{akash-etal-2023-shironaam,
  address = {Dubrovnik, Croatia},
  author = {Akash, Abu Ubaida  and
Nayeem, Mir Tafseer  and
Shohan, Faisal Tareque  and
Islam, Tanvir},
  booktitle = {Proceedings of the 17th Conference of the European Chapter of the Association for Computational Linguistics},
  month = may,
  pages = {52--67},
  publisher = {Association for Computational Linguistics},
  title = {Shironaam: {B}engali News Headline Generation using Auxiliary Information},
  url = {https://aclanthology.org/2023.eacl-main.4},
  year = {2023},
}
""",
    )

    def dataset_transform(self) -> None:
        self.dataset = self.stratified_subsampling(
            self.dataset, seed=self.seed, splits=["test"]
        )



from mteb.models.model_meta import ModelMeta
from mteb.models.sentence_transformer_wrapper import sentence_transformers_loader

xyz_zh_datasets = {
    "BQ",
    "LCQMC",
    "PAWSX",
    "STS-B",
    "DuRetrieval",
    "AFQMC",
    "Cmnli",
    "Ocnli",
    "T2Retrieval",
    "T2Reranking",
    "MMarcoReranking",
    "CMedQAv2-reranking",
}
# not in mteb
# "Covid-News":"NCPPolicies_train",
# "cMedQA":"https://github.com/zhangsheng93/cMedQA",
# "Multi-CPR":"http://github.com/Alibaba-NLP/Multi-CPR",
# "retrieval_data_llm":"https://huggingface.co/datasets/infgrad/retrieval_data_llm",
# "Huatuo26M-Lite":"https://huggingface.co/datasets/FreedomIntelligence/Huatuo26M-Lite"

xyz_embedding = ModelMeta(
    name="fangxq/XYZ-embedding",
    languages=["zho-Hans"],
    loader=sentence_transformers_loader,
    open_weights=True,
    revision="4004120220b99baea764a1d3508427248ac3bccf",
    release_date="2024-09-13",
    n_parameters=326000000,
    memory_usage_mb=1242,
    max_tokens=512,
    embed_dim=768,
    license="mit",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers"],
    reference="https://huggingface.co/fangxq/XYZ-embedding",
    use_instructions=False,
    training_datasets=xyz_zh_datasets,
    public_training_code=None,
    public_training_data=None,
)



import logging
import sys
from collections import defaultdict
from copy import copy

import datasets
from datasets import Dataset

from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.retrieval_dataset_loaders import RetrievalSplitData

if sys.version_info >= (3, 13):
    from warnings import deprecated
else:
    from typing_extensions import deprecated

logger = logging.getLogger(__name__)

OLD_FORMAT_RERANKING_TASKS = []


@deprecated(
    "This class is deprecated and will be removed in future versions. Please use the updated retrieval tasks instead. "
    "You can add your task name to mteb.abstasks.text.reranking.OLD_FORMAT_RERANKING_TASKS to load it in new format. "
    "You can reupload it using `task.push_dataset_to_hub('your/repository')` after loading the data."
)
class AbsTaskReranking(AbsTaskRetrieval):
    """Reranking task class.

    Warning: Deprecated
        This class is deprecated and will be removed in future versions. Please use the updated retrieval tasks instead.
        You can add your task name to mteb.abstasks.text.reranking.OLD_FORMAT_RERANKING_TASKS to load it in new format.
        You can reupload it using `task.push_dataset_to_hub('your/repository')` after loading the data.
        For dataformat and other information, see [AbsTaskRetrieval][mteb.abstasks.retrieval.AbsTaskRetrieval].
    """

    def load_data(self) -> None:
        """Load the dataset."""
        if self.data_loaded:
            return

        if self.metadata.name in OLD_FORMAT_RERANKING_TASKS:
            self.transform_old_dataset_format()
        else:
            # use AbsTaskRetrieval default to load the data
            return super().load_data()

    def _process_example(self, example: dict, split: str, query_idx: int) -> dict:
        """Process a single example from the dataset.

        Args:
            example: A single example from the dataset containing 'query', 'positive', and 'negative' fields.
            split: The dataset split (e.g., 'train', 'validation', 'test').
            query_idx: The index of the query in the dataset split.

        Returns:
            A dictionary containing the processed example with query_id, query text, document ids, document texts, and relevance scores.
        """
        query = example["query"]
        positive_docs = example["positive"]
        negative_docs = example["negative"]

        query_id = f"{split}_query{query_idx}"

        # Initialize the structures for this example
        example_data = {
            "query_id": query_id,
            "query": query,
            "doc_ids": [],
            "doc_texts": [],
            "relevance_scores": [],
        }

        for i, pos_doc in enumerate(positive_docs):
            # format i as a five digit number
            formatted_i = str(i).zfill(5)
            # have "a" in front so that positives are first, then negatives
            #   this shouldn't matter except for ties, and the previous reranking results
            #   had the positives first
            doc_id = f"apositive_{query_id}_{formatted_i}"
            example_data["doc_ids"].append(doc_id)
            example_data["doc_texts"].append(pos_doc)
            example_data["relevance_scores"].append(1)

        for i, neg_doc in enumerate(negative_docs):
            formatted_i = str(i).zfill(5)
            doc_id = f"negative_{query_id}_{formatted_i}"
            example_data["doc_ids"].append(doc_id)
            example_data["doc_texts"].append(neg_doc)
            example_data["relevance_scores"].append(0)

        return example_data

    def transform_old_dataset_format(self, given_dataset: Dataset | None = None):
        """Transform the old format to the new format using HF datasets mapping. This is a one-time transformation for datasets which are in the old format.

        Args:
            given_dataset (Dataset, optional): The dataset to transform. Defaults to None. This is helpful for some older datasets which are loaded with custom code, but need to be transformed still.
        """
        if self.metadata.name not in OLD_FORMAT_RERANKING_TASKS:
            return

        logging.info(
            f"Transforming old format to standard format for {self.metadata.name}"
        )

        given_dataset = copy(given_dataset)
        self.dataset = defaultdict(lambda: defaultdict(dict))

        hf_subsets = self.hf_subsets

        for hf_subset in hf_subsets:
            if given_dataset:
                cur_dataset = given_dataset
                if hf_subset in cur_dataset:
                    cur_dataset = cur_dataset[hf_subset]
            elif "name" in self.metadata.dataset:
                cur_dataset = datasets.load_dataset(**self.metadata.dataset)  # type: ignore
                assert hf_subset == "default", (
                    f"Only default subset is supported for {self.metadata.name} since `name` is given in the metadata."
                )
            else:
                cur_dataset = datasets.load_dataset(
                    **self.metadata.dataset, name=hf_subset
                )  # type: ignore

            for split in cur_dataset:
                corpus = []
                queries = []
                relevant_docs = defaultdict(dict)
                top_ranked = defaultdict(list)

                # Create an enumerated dataset to pass indices
                enumerated_dataset = Dataset.from_dict(
                    {
                        "index": range(len(cur_dataset[split])),
                        "query": cur_dataset[split]["query"],
                        "positive": cur_dataset[split]["positive"],
                        "negative": cur_dataset[split]["negative"],
                    }
                )

                # first, filter out the ones that have no positive or no negatives
                enumerated_dataset = enumerated_dataset.filter(
                    lambda example: len(example["positive"]) > 0
                    and len(example["negative"]) > 0
                )

                logger.info(
                    f"Filtered out {len(cur_dataset[split]) - len(enumerated_dataset)} examples with no positive or no negative examples. {len(enumerated_dataset)} examples remaining."
                )

                # Map the transformation function over the dataset
                processed_dataset = enumerated_dataset.map(
                    lambda example, idx: self._process_example(example, split, idx),
                    with_indices=True,
                    remove_columns=enumerated_dataset.column_names,
                )

                # Populate the data structures
                for item in processed_dataset:
                    query_id = item["query_id"]
                    queries.append({"id": query_id, "text": item["query"]})

                    # Add documents and relevance information
                    for doc_id, doc_text, relevance in zip(
                        item["doc_ids"], item["doc_texts"], item["relevance_scores"]
                    ):
                        corpus.append(
                            {
                                "title": "",
                                "text": doc_text,
                                "id": doc_id,
                            }
                        )
                        top_ranked[query_id].append(doc_id)
                        relevant_docs[query_id][doc_id] = relevance

                self.dataset[hf_subset][split] = RetrievalSplitData(
                    corpus=Dataset.from_list(corpus),
                    queries=Dataset.from_list(queries),
                    relevant_docs=relevant_docs,
                    top_ranked=top_ranked,
                )
        self.data_loaded = True



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class LegalQANLRetrieval(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="LegalQANLRetrieval",
        description="To this end, we create and publish a Dutch legal QA dataset, consisting of question-answer pairs "
        "with attributions to Dutch law articles.",
        reference="https://aclanthology.org/2024.nllp-1.12/",
        dataset={
            "path": "clips/mteb-nl-legalqa-pr",
            "revision": "8f593522dfbe7ec07055ca9d38a700e7643d3882",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["nld-Latn"],
        main_score="ndcg_at_10",
        date=("2021-05-01", "2021-08-26"),
        domains=["Legal", "Written"],
        task_subtypes=[],
        license="cc-by-nc-sa-4.0",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{redelaar2024attributed,
  author = {Redelaar, Felicia and Van Drie, Romy and Verberne, Suzan and De Boer, Maaike},
  booktitle = {Proceedings of the natural legal language processing workshop 2024},
  pages = {154--165},
  title = {Attributed Question Answering for Preconditions in the Dutch Law},
  year = {2024},
}
""",
        prompt={
            "query": "Gegeven een juridische vraag, haal documenten op die kunnen helpen bij het beantwoorden van de vraag"
        },
    )
